# pylint:disable=line-too-long
import logging

from ...sim_type import SimTypeFunction,     SimTypeShort, SimTypeInt, SimTypeLong, SimTypeLongLong, SimTypeDouble, SimTypeFloat,     SimTypePointer,     SimTypeChar,     SimStruct,     SimTypeFixedSizeArray,     SimTypeBottom,     SimUnion,     SimTypeBool
from ...calling_conventions import SimCCStdcall, SimCCMicrosoftAMD64
from .. import SIM_PROCEDURES as P
from . import SimLibrary


_l = logging.getLogger(name=__name__)


lib = SimLibrary()
lib.set_default_cc('X86', SimCCStdcall)
lib.set_default_cc('AMD64', SimCCMicrosoftAMD64)
lib.set_library_names("rpcns4.dll")
prototypes = \
    {
        #
        'RpcIfIdVectorFree': SimTypeFunction([SimTypePointer(SimTypePointer(SimStruct({"Count": SimTypeInt(signed=False, label="UInt32"), "IfId": SimTypePointer(SimTypePointer(SimStruct({"Uuid": SimTypeBottom(label="Guid"), "VersMajor": SimTypeShort(signed=False, label="UInt16"), "VersMinor": SimTypeShort(signed=False, label="UInt16")}, name="RPC_IF_ID", pack=False, align=None), offset=0), offset=0)}, name="RPC_IF_ID_VECTOR", pack=False, align=None), offset=0), offset=0)], SimTypeInt(signed=False, label="RPC_STATUS"), arg_names=["IfIdVector"]),
        #
        'RpcNsBindingExportA': SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimStruct({"Count": SimTypeInt(signed=False, label="UInt32"), "BindingH": SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)}, name="RPC_BINDING_VECTOR", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"Count": SimTypeInt(signed=False, label="UInt32"), "Uuid": SimTypePointer(SimTypePointer(SimTypeBottom(label="Guid"), offset=0), offset=0)}, name="UUID_VECTOR", pack=False, align=None), offset=0)], SimTypeInt(signed=False, label="RPC_STATUS"), arg_names=["EntryNameSyntax", "EntryName", "IfSpec", "BindingVec", "ObjectUuidVec"]),
        #
        'RpcNsBindingUnexportA': SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimStruct({"Count": SimTypeInt(signed=False, label="UInt32"), "Uuid": SimTypePointer(SimTypePointer(SimTypeBottom(label="Guid"), offset=0), offset=0)}, name="UUID_VECTOR", pack=False, align=None), offset=0)], SimTypeInt(signed=False, label="RPC_STATUS"), arg_names=["EntryNameSyntax", "EntryName", "IfSpec", "ObjectUuidVec"]),
        #
        'RpcNsBindingExportW': SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimStruct({"Count": SimTypeInt(signed=False, label="UInt32"), "BindingH": SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)}, name="RPC_BINDING_VECTOR", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"Count": SimTypeInt(signed=False, label="UInt32"), "Uuid": SimTypePointer(SimTypePointer(SimTypeBottom(label="Guid"), offset=0), offset=0)}, name="UUID_VECTOR", pack=False, align=None), offset=0)], SimTypeInt(signed=False, label="RPC_STATUS"), arg_names=["EntryNameSyntax", "EntryName", "IfSpec", "BindingVec", "ObjectUuidVec"]),
        #
        'RpcNsBindingUnexportW': SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimStruct({"Count": SimTypeInt(signed=False, label="UInt32"), "Uuid": SimTypePointer(SimTypePointer(SimTypeBottom(label="Guid"), offset=0), offset=0)}, name="UUID_VECTOR", pack=False, align=None), offset=0)], SimTypeInt(signed=False, label="RPC_STATUS"), arg_names=["EntryNameSyntax", "EntryName", "IfSpec", "ObjectUuidVec"]),
        #
        'RpcNsBindingExportPnPA': SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimStruct({"Count": SimTypeInt(signed=False, label="UInt32"), "Uuid": SimTypePointer(SimTypePointer(SimTypeBottom(label="Guid"), offset=0), offset=0)}, name="UUID_VECTOR", pack=False, align=None), offset=0)], SimTypeInt(signed=False, label="RPC_STATUS"), arg_names=["EntryNameSyntax", "EntryName", "IfSpec", "ObjectVector"]),
        #
        'RpcNsBindingUnexportPnPA': SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimStruct({"Count": SimTypeInt(signed=False, label="UInt32"), "Uuid": SimTypePointer(SimTypePointer(SimTypeBottom(label="Guid"), offset=0), offset=0)}, name="UUID_VECTOR", pack=False, align=None), offset=0)], SimTypeInt(signed=False, label="RPC_STATUS"), arg_names=["EntryNameSyntax", "EntryName", "IfSpec", "ObjectVector"]),
        #
        'RpcNsBindingExportPnPW': SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimStruct({"Count": SimTypeInt(signed=False, label="UInt32"), "Uuid": SimTypePointer(SimTypePointer(SimTypeBottom(label="Guid"), offset=0), offset=0)}, name="UUID_VECTOR", pack=False, align=None), offset=0)], SimTypeInt(signed=False, label="RPC_STATUS"), arg_names=["EntryNameSyntax", "EntryName", "IfSpec", "ObjectVector"]),
        #
        'RpcNsBindingUnexportPnPW': SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimStruct({"Count": SimTypeInt(signed=False, label="UInt32"), "Uuid": SimTypePointer(SimTypePointer(SimTypeBottom(label="Guid"), offset=0), offset=0)}, name="UUID_VECTOR", pack=False, align=None), offset=0)], SimTypeInt(signed=False, label="RPC_STATUS"), arg_names=["EntryNameSyntax", "EntryName", "IfSpec", "ObjectVector"]),
        #
        'RpcNsBindingLookupBeginA': SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=False, label="RPC_STATUS"), arg_names=["EntryNameSyntax", "EntryName", "IfSpec", "ObjUuid", "BindingMaxCount", "LookupContext"]),
        #
        'RpcNsBindingLookupBeginW': SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=False, label="RPC_STATUS"), arg_names=["EntryNameSyntax", "EntryName", "IfSpec", "ObjUuid", "BindingMaxCount", "LookupContext"]),
        #
        'RpcNsBindingLookupNext': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimStruct({"Count": SimTypeInt(signed=False, label="UInt32"), "BindingH": SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)}, name="RPC_BINDING_VECTOR", pack=False, align=None), offset=0), offset=0)], SimTypeInt(signed=False, label="RPC_STATUS"), arg_names=["LookupContext", "BindingVec"]),
        #
        'RpcNsBindingLookupDone': SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=False, label="RPC_STATUS"), arg_names=["LookupContext"]),
        #
        'RpcNsGroupDeleteA': SimTypeFunction([SimTypeInt(signed=False, label="GROUP_NAME_SYNTAX"), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeInt(signed=False, label="RPC_STATUS"), arg_names=["GroupNameSyntax", "GroupName"]),
        #
        'RpcNsGroupMbrAddA': SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeInt(signed=False, label="RPC_STATUS"), arg_names=["GroupNameSyntax", "GroupName", "MemberNameSyntax", "MemberName"]),
        #
        'RpcNsGroupMbrRemoveA': SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeInt(signed=False, label="RPC_STATUS"), arg_names=["GroupNameSyntax", "GroupName", "MemberNameSyntax", "MemberName"]),
        #
        'RpcNsGroupMbrInqBeginA': SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=False, label="RPC_STATUS"), arg_names=["GroupNameSyntax", "GroupName", "MemberNameSyntax", "InquiryContext"]),
        #
        'RpcNsGroupMbrInqNextA': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)], SimTypeInt(signed=False, label="RPC_STATUS"), arg_names=["InquiryContext", "MemberName"]),
        #
        'RpcNsGroupDeleteW': SimTypeFunction([SimTypeInt(signed=False, label="GROUP_NAME_SYNTAX"), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)], SimTypeInt(signed=False, label="RPC_STATUS"), arg_names=["GroupNameSyntax", "GroupName"]),
        #
        'RpcNsGroupMbrAddW': SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)], SimTypeInt(signed=False, label="RPC_STATUS"), arg_names=["GroupNameSyntax", "GroupName", "MemberNameSyntax", "MemberName"]),
        #
        'RpcNsGroupMbrRemoveW': SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)], SimTypeInt(signed=False, label="RPC_STATUS"), arg_names=["GroupNameSyntax", "GroupName", "MemberNameSyntax", "MemberName"]),
        #
        'RpcNsGroupMbrInqBeginW': SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=False, label="RPC_STATUS"), arg_names=["GroupNameSyntax", "GroupName", "MemberNameSyntax", "InquiryContext"]),
        #
        'RpcNsGroupMbrInqNextW': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), offset=0)], SimTypeInt(signed=False, label="RPC_STATUS"), arg_names=["InquiryContext", "MemberName"]),
        #
        'RpcNsGroupMbrInqDone': SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=False, label="RPC_STATUS"), arg_names=["InquiryContext"]),
        #
        'RpcNsProfileDeleteA': SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeInt(signed=False, label="RPC_STATUS"), arg_names=["ProfileNameSyntax", "ProfileName"]),
        #
        'RpcNsProfileEltAddA': SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimStruct({"Uuid": SimTypeBottom(label="Guid"), "VersMajor": SimTypeShort(signed=False, label="UInt16"), "VersMinor": SimTypeShort(signed=False, label="UInt16")}, name="RPC_IF_ID", pack=False, align=None), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeInt(signed=False, label="RPC_STATUS"), arg_names=["ProfileNameSyntax", "ProfileName", "IfId", "MemberNameSyntax", "MemberName", "Priority", "Annotation"]),
        #
        'RpcNsProfileEltRemoveA': SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimStruct({"Uuid": SimTypeBottom(label="Guid"), "VersMajor": SimTypeShort(signed=False, label="UInt16"), "VersMinor": SimTypeShort(signed=False, label="UInt16")}, name="RPC_IF_ID", pack=False, align=None), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeInt(signed=False, label="RPC_STATUS"), arg_names=["ProfileNameSyntax", "ProfileName", "IfId", "MemberNameSyntax", "MemberName"]),
        #
        'RpcNsProfileEltInqBeginA': SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimStruct({"Uuid": SimTypeBottom(label="Guid"), "VersMajor": SimTypeShort(signed=False, label="UInt16"), "VersMinor": SimTypeShort(signed=False, label="UInt16")}, name="RPC_IF_ID", pack=False, align=None), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=False, label="RPC_STATUS"), arg_names=["ProfileNameSyntax", "ProfileName", "InquiryType", "IfId", "VersOption", "MemberNameSyntax", "MemberName", "InquiryContext"]),
        #
        'RpcNsProfileEltInqNextA': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimStruct({"Uuid": SimTypeBottom(label="Guid"), "VersMajor": SimTypeShort(signed=False, label="UInt16"), "VersMinor": SimTypeShort(signed=False, label="UInt16")}, name="RPC_IF_ID", pack=False, align=None), offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)], SimTypeInt(signed=False, label="RPC_STATUS"), arg_names=["InquiryContext", "IfId", "MemberName", "Priority", "Annotation"]),
        #
        'RpcNsProfileDeleteW': SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)], SimTypeInt(signed=False, label="RPC_STATUS"), arg_names=["ProfileNameSyntax", "ProfileName"]),
        #
        'RpcNsProfileEltAddW': SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimStruct({"Uuid": SimTypeBottom(label="Guid"), "VersMajor": SimTypeShort(signed=False, label="UInt16"), "VersMinor": SimTypeShort(signed=False, label="UInt16")}, name="RPC_IF_ID", pack=False, align=None), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)], SimTypeInt(signed=False, label="RPC_STATUS"), arg_names=["ProfileNameSyntax", "ProfileName", "IfId", "MemberNameSyntax", "MemberName", "Priority", "Annotation"]),
        #
        'RpcNsProfileEltRemoveW': SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimStruct({"Uuid": SimTypeBottom(label="Guid"), "VersMajor": SimTypeShort(signed=False, label="UInt16"), "VersMinor": SimTypeShort(signed=False, label="UInt16")}, name="RPC_IF_ID", pack=False, align=None), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)], SimTypeInt(signed=False, label="RPC_STATUS"), arg_names=["ProfileNameSyntax", "ProfileName", "IfId", "MemberNameSyntax", "MemberName"]),
        #
        'RpcNsProfileEltInqBeginW': SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimStruct({"Uuid": SimTypeBottom(label="Guid"), "VersMajor": SimTypeShort(signed=False, label="UInt16"), "VersMinor": SimTypeShort(signed=False, label="UInt16")}, name="RPC_IF_ID", pack=False, align=None), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=False, label="RPC_STATUS"), arg_names=["ProfileNameSyntax", "ProfileName", "InquiryType", "IfId", "VersOption", "MemberNameSyntax", "MemberName", "InquiryContext"]),
        #
        'RpcNsProfileEltInqNextW': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimStruct({"Uuid": SimTypeBottom(label="Guid"), "VersMajor": SimTypeShort(signed=False, label="UInt16"), "VersMinor": SimTypeShort(signed=False, label="UInt16")}, name="RPC_IF_ID", pack=False, align=None), offset=0), SimTypePointer(SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), offset=0)], SimTypeInt(signed=False, label="RPC_STATUS"), arg_names=["InquiryContext", "IfId", "MemberName", "Priority", "Annotation"]),
        #
        'RpcNsProfileEltInqDone': SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=False, label="RPC_STATUS"), arg_names=["InquiryContext"]),
        #
        'RpcNsEntryObjectInqBeginA': SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=False, label="RPC_STATUS"), arg_names=["EntryNameSyntax", "EntryName", "InquiryContext"]),
        #
        'RpcNsEntryObjectInqBeginW': SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=False, label="RPC_STATUS"), arg_names=["EntryNameSyntax", "EntryName", "InquiryContext"]),
        #
        'RpcNsEntryObjectInqNext': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0)], SimTypeInt(signed=False, label="RPC_STATUS"), arg_names=["InquiryContext", "ObjUuid"]),
        #
        'RpcNsEntryObjectInqDone': SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=False, label="RPC_STATUS"), arg_names=["InquiryContext"]),
        #
        'RpcNsEntryExpandNameA': SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)], SimTypeInt(signed=False, label="RPC_STATUS"), arg_names=["EntryNameSyntax", "EntryName", "ExpandedName"]),
        #
        'RpcNsMgmtBindingUnexportA': SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimStruct({"Uuid": SimTypeBottom(label="Guid"), "VersMajor": SimTypeShort(signed=False, label="UInt16"), "VersMinor": SimTypeShort(signed=False, label="UInt16")}, name="RPC_IF_ID", pack=False, align=None), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimStruct({"Count": SimTypeInt(signed=False, label="UInt32"), "Uuid": SimTypePointer(SimTypePointer(SimTypeBottom(label="Guid"), offset=0), offset=0)}, name="UUID_VECTOR", pack=False, align=None), offset=0)], SimTypeInt(signed=False, label="RPC_STATUS"), arg_names=["EntryNameSyntax", "EntryName", "IfId", "VersOption", "ObjectUuidVec"]),
        #
        'RpcNsMgmtEntryCreateA': SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeInt(signed=False, label="RPC_STATUS"), arg_names=["EntryNameSyntax", "EntryName"]),
        #
        'RpcNsMgmtEntryDeleteA': SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeInt(signed=False, label="RPC_STATUS"), arg_names=["EntryNameSyntax", "EntryName"]),
        #
        'RpcNsMgmtEntryInqIfIdsA': SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypePointer(SimStruct({"Count": SimTypeInt(signed=False, label="UInt32"), "IfId": SimTypePointer(SimTypePointer(SimStruct({"Uuid": SimTypeBottom(label="Guid"), "VersMajor": SimTypeShort(signed=False, label="UInt16"), "VersMinor": SimTypeShort(signed=False, label="UInt16")}, name="RPC_IF_ID", pack=False, align=None), offset=0), offset=0)}, name="RPC_IF_ID_VECTOR", pack=False, align=None), offset=0), offset=0)], SimTypeInt(signed=False, label="RPC_STATUS"), arg_names=["EntryNameSyntax", "EntryName", "IfIdVec"]),
        #
        'RpcNsMgmtHandleSetExpAge': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="RPC_STATUS"), arg_names=["NsHandle", "ExpirationAge"]),
        #
        'RpcNsMgmtInqExpAge': SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="RPC_STATUS"), arg_names=["ExpirationAge"]),
        #
        'RpcNsMgmtSetExpAge': SimTypeFunction([SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="RPC_STATUS"), arg_names=["ExpirationAge"]),
        #
        'RpcNsEntryExpandNameW': SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), offset=0)], SimTypeInt(signed=False, label="RPC_STATUS"), arg_names=["EntryNameSyntax", "EntryName", "ExpandedName"]),
        #
        'RpcNsMgmtBindingUnexportW': SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimStruct({"Uuid": SimTypeBottom(label="Guid"), "VersMajor": SimTypeShort(signed=False, label="UInt16"), "VersMinor": SimTypeShort(signed=False, label="UInt16")}, name="RPC_IF_ID", pack=False, align=None), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimStruct({"Count": SimTypeInt(signed=False, label="UInt32"), "Uuid": SimTypePointer(SimTypePointer(SimTypeBottom(label="Guid"), offset=0), offset=0)}, name="UUID_VECTOR", pack=False, align=None), offset=0)], SimTypeInt(signed=False, label="RPC_STATUS"), arg_names=["EntryNameSyntax", "EntryName", "IfId", "VersOption", "ObjectUuidVec"]),
        #
        'RpcNsMgmtEntryCreateW': SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)], SimTypeInt(signed=False, label="RPC_STATUS"), arg_names=["EntryNameSyntax", "EntryName"]),
        #
        'RpcNsMgmtEntryDeleteW': SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)], SimTypeInt(signed=False, label="RPC_STATUS"), arg_names=["EntryNameSyntax", "EntryName"]),
        #
        'RpcNsMgmtEntryInqIfIdsW': SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypePointer(SimStruct({"Count": SimTypeInt(signed=False, label="UInt32"), "IfId": SimTypePointer(SimTypePointer(SimStruct({"Uuid": SimTypeBottom(label="Guid"), "VersMajor": SimTypeShort(signed=False, label="UInt16"), "VersMinor": SimTypeShort(signed=False, label="UInt16")}, name="RPC_IF_ID", pack=False, align=None), offset=0), offset=0)}, name="RPC_IF_ID_VECTOR", pack=False, align=None), offset=0), offset=0)], SimTypeInt(signed=False, label="RPC_STATUS"), arg_names=["EntryNameSyntax", "EntryName", "IfIdVec"]),
        #
        'RpcNsBindingImportBeginA': SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=False, label="RPC_STATUS"), arg_names=["EntryNameSyntax", "EntryName", "IfSpec", "ObjUuid", "ImportContext"]),
        #
        'RpcNsBindingImportBeginW': SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=False, label="RPC_STATUS"), arg_names=["EntryNameSyntax", "EntryName", "IfSpec", "ObjUuid", "ImportContext"]),
        #
        'RpcNsBindingImportNext': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=False, label="RPC_STATUS"), arg_names=["ImportContext", "Binding"]),
        #
        'RpcNsBindingImportDone': SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=False, label="RPC_STATUS"), arg_names=["ImportContext"]),
        #
        'RpcNsBindingSelect': SimTypeFunction([SimTypePointer(SimStruct({"Count": SimTypeInt(signed=False, label="UInt32"), "BindingH": SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)}, name="RPC_BINDING_VECTOR", pack=False, align=None), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=False, label="RPC_STATUS"), arg_names=["BindingVec", "Binding"]),
        #
        'I_RpcNsGetBuffer': SimTypeFunction([SimTypePointer(SimStruct({"Handle": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "DataRepresentation": SimTypeInt(signed=False, label="UInt32"), "Buffer": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "BufferLength": SimTypeInt(signed=False, label="UInt32"), "ProcNum": SimTypeInt(signed=False, label="UInt32"), "TransferSyntax": SimTypePointer(SimStruct({"SyntaxGUID": SimTypeBottom(label="Guid"), "SyntaxVersion": SimStruct({"MajorVersion": SimTypeShort(signed=False, label="UInt16"), "MinorVersion": SimTypeShort(signed=False, label="UInt16")}, name="RPC_VERSION", pack=False, align=None)}, name="RPC_SYNTAX_IDENTIFIER", pack=False, align=None), offset=0), "RpcInterfaceInformation": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "ReservedForRuntime": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "ManagerEpv": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "ImportContext": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "RpcFlags": SimTypeInt(signed=False, label="UInt32")}, name="RPC_MESSAGE", pack=False, align=None), offset=0)], SimTypeInt(signed=False, label="RPC_STATUS"), arg_names=["Message"]),
        #
        'I_RpcNsSendReceive': SimTypeFunction([SimTypePointer(SimStruct({"Handle": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "DataRepresentation": SimTypeInt(signed=False, label="UInt32"), "Buffer": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "BufferLength": SimTypeInt(signed=False, label="UInt32"), "ProcNum": SimTypeInt(signed=False, label="UInt32"), "TransferSyntax": SimTypePointer(SimStruct({"SyntaxGUID": SimTypeBottom(label="Guid"), "SyntaxVersion": SimStruct({"MajorVersion": SimTypeShort(signed=False, label="UInt16"), "MinorVersion": SimTypeShort(signed=False, label="UInt16")}, name="RPC_VERSION", pack=False, align=None)}, name="RPC_SYNTAX_IDENTIFIER", pack=False, align=None), offset=0), "RpcInterfaceInformation": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "ReservedForRuntime": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "ManagerEpv": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "ImportContext": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "RpcFlags": SimTypeInt(signed=False, label="UInt32")}, name="RPC_MESSAGE", pack=False, align=None), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=False, label="RPC_STATUS"), arg_names=["Message", "Handle"]),
        #
        'I_RpcNsRaiseException': SimTypeFunction([SimTypePointer(SimStruct({"Handle": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "DataRepresentation": SimTypeInt(signed=False, label="UInt32"), "Buffer": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "BufferLength": SimTypeInt(signed=False, label="UInt32"), "ProcNum": SimTypeInt(signed=False, label="UInt32"), "TransferSyntax": SimTypePointer(SimStruct({"SyntaxGUID": SimTypeBottom(label="Guid"), "SyntaxVersion": SimStruct({"MajorVersion": SimTypeShort(signed=False, label="UInt16"), "MinorVersion": SimTypeShort(signed=False, label="UInt16")}, name="RPC_VERSION", pack=False, align=None)}, name="RPC_SYNTAX_IDENTIFIER", pack=False, align=None), offset=0), "RpcInterfaceInformation": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "ReservedForRuntime": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "ManagerEpv": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "ImportContext": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "RpcFlags": SimTypeInt(signed=False, label="UInt32")}, name="RPC_MESSAGE", pack=False, align=None), offset=0), SimTypeInt(signed=False, label="RPC_STATUS")], SimTypeBottom(label="Void"), arg_names=["Message", "Status"]),
        #
        'I_RpcReBindBuffer': SimTypeFunction([SimTypePointer(SimStruct({"Handle": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "DataRepresentation": SimTypeInt(signed=False, label="UInt32"), "Buffer": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "BufferLength": SimTypeInt(signed=False, label="UInt32"), "ProcNum": SimTypeInt(signed=False, label="UInt32"), "TransferSyntax": SimTypePointer(SimStruct({"SyntaxGUID": SimTypeBottom(label="Guid"), "SyntaxVersion": SimStruct({"MajorVersion": SimTypeShort(signed=False, label="UInt16"), "MinorVersion": SimTypeShort(signed=False, label="UInt16")}, name="RPC_VERSION", pack=False, align=None)}, name="RPC_SYNTAX_IDENTIFIER", pack=False, align=None), offset=0), "RpcInterfaceInformation": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "ReservedForRuntime": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "ManagerEpv": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "ImportContext": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "RpcFlags": SimTypeInt(signed=False, label="UInt32")}, name="RPC_MESSAGE", pack=False, align=None), offset=0)], SimTypeInt(signed=False, label="RPC_STATUS"), arg_names=["Message"]),
    }

lib.set_prototypes(prototypes)



#!/usr/bin/env python3
# pylint: disable=missing-class-docstring,no-self-use
__package__ = __package__ or "tests.exploration_techniques"  # pylint:disable=redefined-builtin

import os
import unittest

import angr
import psutil

from ..common import bin_location


test_location = os.path.join(bin_location, "tests")


class TestMemoryWatcher(unittest.TestCase):
    def test_memory_watcher(self):
        binary = os.path.join(test_location, "x86_64", "veritesting_a")
        proj = angr.Project(binary, auto_load_libs=False)
        simgr = proj.factory.simulation_manager()

        memory_watcher = angr.exploration_techniques.MemoryWatcher()
        simgr.use_technique(memory_watcher)

        # Initially build some paths
        while len(simgr.active) < 32 and simgr.active != []:
            simgr.step()

        # Something else went wrong..
        assert simgr.active != []

        # Set fake that memory watcher believes we're too low on memory
        memory_watcher.min_memory = psutil.virtual_memory().total

        previous_active = len(simgr.active)

        # Step once to move things over
        simgr.step()

        assert simgr.active == []
        assert len(getattr(simgr, memory_watcher.memory_stash)) == previous_active


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python3
# pylint: disable=missing-class-docstring,no-self-use,line-too-long
__package__ = __package__ or "tests.state_plugins.solver"  # pylint:disable=redefined-builtin

import unittest

import angr

from ...common import broken


class TestSymbolic(unittest.TestCase):
    # @broken
    # def test_symvalue(self):
    #   # concrete symvalue
    #   zero = SimValue(se.BVV(0, 64))
    #   assert not zero.is_symbolic()
    #   assert zero.eval() == 0
    #
    #   # symbolic symvalue
    #   x = se.BVS('x', 64)
    #   sym = SimValue(x, constraints = [ x > 100, x < 200 ])
    #   assert sym.is_symbolic()
    #   assert sym.min_int() == 101
    #   assert sym.max_int() == 199
    #   assert sym.eval_upto(99) == range(101, 200)

    def test_concretization_strategies(self):
        initial_memory = {0: b"A", 1: b"B", 2: b"C", 3: b"D"}

        s = angr.SimState(arch="AMD64", dict_memory_backer=initial_memory)

        # sanity check
        assert s.solver.eval_upto(s.memory.load(3, size=1), 2, cast_to=bytes) == [b"D"]

        x = s.solver.BVS("x", s.arch.bits)
        s.add_constraints(x >= 1)
        s.add_constraints(x <= 3)

        ss = s.copy()
        assert tuple(sorted(ss.solver.eval_upto(ss.memory.load(x, 1), 10, cast_to=bytes))) == (b"B", b"C", b"D")

        ss = s.copy()
        x = s.solver.BVS("x", s.arch.bits)
        s.add_constraints(x >= 1)
        ss.options.add(angr.options.CONSERVATIVE_READ_STRATEGY)
        ss.memory._create_default_read_strategies()
        assert "symbolic" in next(iter(ss.memory.load(x, 1).variables))

    # def test_concretization(self):
    #   s = angr.SimState(arch="AMD64", mode="symbolic")
    #   dst = s.solver.BVV(0x41424300, 32)
    #   dst_addr = s.solver.BVV(0x1000, 64)
    #   s.memory.store(dst_addr, dst, 4)

    #   print "MEM KEYS", s.memory.mem.keys()
    #   print "REG KEYS", s.registers.mem.keys()

    #   print "TO NATIVE..."
    #   s.set_native(True)
    #   print "... done"

    #   vv = s.native_env.vexecute(pyvex.IRExpr.Load("Iend_BE", "Ity_I32",
    #                              pyvex.IRExpr.Const(pyvex.IRConst.U64(0x1000))))
    #   assert vv.str[:4] == 'ABC\x00'
    #   s.native_env.vexecute(pyvex.IRSB(bytes='\xb8\x41\x42\x43\x44'))

    #   #import IPython; IPython.embed()
    #   print "FROM NATIVE..."
    #   s.set_native(False)
    #   print "... done"

    #   assert s.reg_value(16).solver.eval() == 0x44434241
    #   print "YEAH"

    @broken
    def test_symbolic_write(self):
        s = angr.SimState(arch="AMD64", mode="symbolic")

        addr = s.solver.BVS("addr", 64)
        s.add_constraints(s.solver.Or(addr == 10, addr == 20, addr == 30))
        assert len(s.solver.eval_upto(addr, 10)) == 3

        s.memory.store(10, s.solver.BVV(1, 8))
        s.memory.store(20, s.solver.BVV(2, 8))
        s.memory.store(30, s.solver.BVV(3, 8))

        assert s.solver.unique(s.memory.load(10, 1))
        assert s.solver.unique(s.memory.load(20, 1))
        assert s.solver.unique(s.memory.load(30, 1))

        # print "CONSTRAINTS BEFORE:", s.constraints._solver.constraints
        # s.memory.store(addr, s.solver.BVV(255, 8), strategy=['symbolic','any'], limit=100)
        s.memory.store(addr, s.solver.BVV(255, 8))
        assert s.satisfiable()

        assert len(s.solver.eval_upto(addr, 10)) == 3
        assert s.solver.eval_upto(s.memory.load(10, 1), 3) == [1, 255]
        assert s.solver.eval_upto(s.memory.load(20, 1), 3) == [2, 255]
        assert s.solver.eval_upto(s.memory.load(30, 1), 3) == [3, 255]
        assert len(s.solver.eval_upto(addr, 10)) == 3

        # see if it works when constraining the write address
        sa = s.copy()
        sa.add_constraints(addr == 20)
        assert sa.satisfiable()
        assert sa.solver.eval_upto(sa.memory.load(10, 1), 3) == [1]
        assert sa.solver.eval_upto(sa.memory.load(20, 1), 3) == [255]
        assert sa.solver.eval_upto(sa.memory.load(30, 1), 3) == [3]
        assert sa.solver.eval_upto(addr, 10) == [20]

        # see if it works when constraining a value to the written one
        sv = s.copy()
        sv.add_constraints(sv.memory.load(30, 1) == 255)
        assert sv.satisfiable()
        assert sv.solver.eval_upto(sv.memory.load(10, 1), 3) == [1]
        assert sv.solver.eval_upto(sv.memory.load(20, 1), 3) == [2]
        assert sv.solver.eval_upto(sv.memory.load(30, 1), 3) == [255]
        assert sv.solver.eval_upto(addr, 10) == [30]

        # see if it works when constraining a value to the unwritten one
        sv = s.copy()
        sv.add_constraints(sv.memory.load(30, 1) == 3)
        assert sv.satisfiable()
        assert sv.solver.eval_upto(sv.memory.load(10, 1), 3) == [1, 255]
        assert sv.solver.eval_upto(sv.memory.load(20, 1), 3) == [2, 255]
        assert sv.solver.eval_upto(sv.memory.load(30, 1), 3) == [3]
        assert sv.solver.eval_upto(addr, 10) == [10, 20]

        s = angr.SimState(arch="AMD64", mode="symbolic")
        s.memory.store(0, s.solver.BVV(0x4141414141414141, 64))
        length = s.solver.BVS("length", 32)
        # s.memory.store(0, s.solver.BVV(0x4242424242424242, 64), symbolic_length=length)
        s.memory.store(0, s.solver.BVV(0x4242424242424242, 64))

        for i in range(8):
            ss = s.copy()
            ss.add_constraints(length == i)
            assert ss.solver.eval(s.memory.load(0, 8), cast_to=bytes) == b"B" * i + b"A" * (8 - i)

    def test_unsat_core(self):
        s = angr.SimState(arch="AMD64", mode="symbolic", add_options={angr.options.CONSTRAINT_TRACKING_IN_SOLVER})
        x = s.solver.BVS("x", 32)
        s.add_constraints(s.solver.BVV(0, 32) == x)
        s.add_constraints(s.solver.BVV(1, 32) == x)

        assert not s.satisfiable()
        unsat_core = s.solver.unsat_core()
        assert len(unsat_core) == 2


if __name__ == "__main__":
    unittest.main()



from enum import Enum
from typing import Union

import ailment

from ...block import Block
from ...knowledge_plugins.functions.function_manager import Function
from ..forward_analysis import FunctionGraphVisitor, SingleNodeGraphVisitor


class SubjectType(Enum):
    Function = 1
    Block = 2
    CallTrace = 3


class Subject:
    _visitor: Union[FunctionGraphVisitor, SingleNodeGraphVisitor]

    def __init__(self, content, func_graph=None, cc=None):
        """
        The thing being analysed, and the way (visitor) to analyse it.

        :param Union[ailment.Block, angr.Block, Function] content:
            Thing to be analysed.
        :param networkx.DiGraph func_graph: Alternative graph for function.graph.
        :param SimCC cc: Calling convention of the function.
        """

        self._content = content

        if isinstance(content, Function):
            self._cc = cc or content.calling_convention
            self._func_graph = func_graph
            self._type = SubjectType.Function
            self._visitor = FunctionGraphVisitor(content, func_graph)
        elif isinstance(content, (ailment.Block, Block)):
            self._type = SubjectType.Block
            self._visitor = SingleNodeGraphVisitor(content)
        else:
            raise TypeError("Unsupported analysis target.")

    @property
    def cc(self):
        if self.type not in (SubjectType.Function, SubjectType.CallTrace):
            raise TypeError("There are no `cc` attribute for <%s>." % self.type)
        return self._cc

    @property
    def content(self):
        return self._content

    @property
    def func_graph(self):
        if self.type is not SubjectType.Function:
            raise TypeError("There are no `func_graph` attribute for <%s>." % self.type)
        return self._func_graph

    @property
    def type(self):
        return self._type

    @property
    def visitor(self) -> Union[FunctionGraphVisitor, SingleNodeGraphVisitor]:
        return self._visitor



#!/usr/bin/env python3
# pylint: disable=missing-class-docstring,no-self-use,line-too-long
__package__ = __package__ or "tests.analyses"  # pylint:disable=redefined-builtin

import os
import unittest

import angr

from ..common import bin_location


test_location = os.path.join(bin_location, "tests")


class TestStaticHooker(unittest.TestCase):
    def test_static_hooker(self):
        test_file = os.path.join(test_location, "x86_64", "static")
        p = angr.Project(test_file, auto_load_libs=False)
        sh = p.analyses.StaticHooker("libc.so.6")

        assert 4197616 in sh.results
        assert type(sh.results[4197616]) is angr.SIM_PROCEDURES["glibc"]["__libc_start_main"]
        assert type(p.hooked_by(4197616)) is angr.SIM_PROCEDURES["glibc"]["__libc_start_main"]


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python3
import unittest

import angr
from angr import SimState, SIM_PROCEDURES
from angr.engines import ProcedureEngine

FAKE_ADDR = 0x100000


# pylint: disable=missing-class-docstring
# pylint: disable=no-self-use
class TestActions(unittest.TestCase):
    def test_procedure_actions(self):
        s = SimState(arch="AMD64", add_options={angr.options.ADD_AUTO_REFS})

        s.registers.store("rbx", 2)
        proc = SIM_PROCEDURES["testing"]["retreg"](reg="rbx")
        succ = ProcedureEngine(None).process(s, procedure=proc)
        rbx = succ.artifacts["procedure"].ret_expr
        assert type(rbx) is angr.state_plugins.SimActionObject
        assert s.solver.eval(rbx) == 2
        assert rbx.reg_deps == {s.arch.registers["rbx"][0]}


if __name__ == "__main__":
    unittest.main()



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: protos/xrefs.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import primitives_pb2 as protos_dot_primitives__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x12protos/xrefs.proto\x12\x0b\x61ngr.protos\x1a\x17protos/primitives.proto\"2\n\x05XRefs\x12)\n\x05xrefs\x18\x01 \x03(\x0b\x32\x1a.angr.protos.CodeReferenceb\x06proto3')



_XREFS = DESCRIPTOR.message_types_by_name['XRefs']
XRefs = _reflection.GeneratedProtocolMessageType('XRefs', (_message.Message,), {
  'DESCRIPTOR' : _XREFS,
  '__module__' : 'protos.xrefs_pb2'
  # @@protoc_insertion_point(class_scope:angr.protos.XRefs)
  })
_sym_db.RegisterMessage(XRefs)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _XREFS._serialized_start=60
  _XREFS._serialized_end=110
# @@protoc_insertion_point(module_scope)



#!/usr/bin/env python3
# pylint: disable=missing-class-docstring,no-self-use,line-too-long
__package__ = __package__ or "tests.serialization"  # pylint:disable=redefined-builtin

import gc
import os
import pickle
import shutil
import tempfile
import unittest

import angr

from ..common import bin_location

test_location = os.path.join(bin_location, "tests")

internaltest_files = [
    "argc_decide",
    "argc_symbol",
    "argv_test",
    "counter",
    "fauxware",
    "fauxware.idb",
    "manysum",
    "pw",
    "strlen",
    "test_arrays",
    "test_division",
    "test_loops",
]
internaltest_arch = ["i386", "armel"]


def internaltest_vfg(p, cfg):
    vfg = p.analyses.VFG(cfg=cfg)
    v = angr.vaults.VaultDict()
    state = v.dumps(vfg)
    vfg2 = v.loads(state)
    assert vfg.final_states == vfg2.final_states
    assert set(vfg.graph.nodes()) == set(vfg2.graph.nodes())


def internaltest_cfg(p):
    state = tempfile.TemporaryFile()

    cfg = p.analyses.CFGEmulated()
    pickle.dump(cfg, state, -1)

    state.seek(0)
    cfg2 = pickle.load(state)
    assert set(cfg.model.nodes()) == set(cfg2.model.nodes())
    assert cfg.unresolvables == cfg2.unresolvables
    assert set(cfg.deadends) == set(cfg2.deadends)

    return cfg


def internaltest_cfgfast(p):
    state = tempfile.TemporaryFile()

    cfg = p.analyses.CFGFast()

    # generate capstone blocks
    main_function = cfg.functions.function(name="main")
    for b in main_function.blocks:
        _ = b.capstone

    pickle.dump(cfg, state, -1)

    state.seek(0)
    cfg2 = pickle.load(state)
    assert set(cfg.model.nodes()) == set(cfg2.model.nodes())


def internaltest_project(fpath):
    tpath = tempfile.mktemp()
    shutil.copy(fpath, tpath)

    p = angr.Project(tpath, auto_load_libs=False)
    state = pickle.dumps(p, -1)
    loaded_p = pickle.loads(state)
    assert p is not loaded_p
    assert p.arch == loaded_p.arch
    assert p.filename == loaded_p.filename
    assert p.entry == loaded_p.entry

    simgr = loaded_p.factory.simulation_manager()
    simgr.run(n=10)
    assert len(simgr.errored) == 0


class TestSerialization(unittest.TestCase):
    def test_analyses(self):
        p = angr.Project(os.path.join(test_location, "i386", "fauxware"), load_options={"auto_load_libs": False})
        cfg = p.analyses.CFG()
        cfb = p.analyses.CFB(kb=cfg.kb)
        vrf = p.analyses.VariableRecoveryFast(p.kb.functions["main"])

        assert len(p.kb.functions) > 0
        assert len(pickle.loads(pickle.dumps(p.kb, -1)).functions) > 0

        state = pickle.dumps((p, cfg, cfb, vrf), -1)
        del p
        del cfg
        del cfb
        del vrf

        gc.collect()

        p, cfg, cfb, vrf = pickle.loads(state)
        assert p.kb is not None
        assert p.kb.functions is not None
        assert cfg.kb is not None
        assert len(p.kb.functions) > 0

    def test_serialization(self):
        for d in internaltest_arch:
            for f in internaltest_files:
                fpath = os.path.join(test_location, d, f)
                if os.path.isfile(fpath) and os.access(fpath, os.X_OK):
                    internaltest_project(fpath)

        p = angr.Project(os.path.join(test_location, "i386", "fauxware"), load_options={"auto_load_libs": False})
        internaltest_cfgfast(p)

        cfg = internaltest_cfg(p)
        internaltest_vfg(p, cfg)


if __name__ == "__main__":
    unittest.main()



from typing import TypeVar, Optional, Iterable

_T = TypeVar("_T")


def unwrap(thing: Optional[_T]) -> _T:
    assert thing is not None, "Tried to unwrap a `None` value"
    return thing


def one(thing: Iterable[_T]) -> _T:
    try:
        (result,) = thing
    except ValueError as e:
        raise AssertionError("Tried to get only value of an iterable with some other number of items") from e
    return result



# pylint:disable=line-too-long
import logging

from ...sim_type import SimTypeFunction,     SimTypeShort, SimTypeInt, SimTypeLong, SimTypeLongLong, SimTypeDouble, SimTypeFloat,     SimTypePointer,     SimTypeChar,     SimStruct,     SimTypeFixedSizeArray,     SimTypeBottom,     SimUnion,     SimTypeBool
from ...calling_conventions import SimCCStdcall, SimCCMicrosoftAMD64
from .. import SIM_PROCEDURES as P
from . import SimLibrary


_l = logging.getLogger(name=__name__)


lib = SimLibrary()
lib.set_default_cc('X86', SimCCStdcall)
lib.set_default_cc('AMD64', SimCCMicrosoftAMD64)
lib.set_library_names("webauthn.dll")
prototypes = \
    {
        #
        'WebAuthNGetApiVersionNumber': SimTypeFunction([], SimTypeInt(signed=False, label="UInt32")),
        #
        'WebAuthNIsUserVerifyingPlatformAuthenticatorAvailable': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pbIsUserVerifyingPlatformAuthenticatorAvailable"]),
        #
        'WebAuthNAuthenticatorMakeCredential': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"dwVersion": SimTypeInt(signed=False, label="UInt32"), "pwszId": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pwszName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pwszIcon": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="WEBAUTHN_RP_ENTITY_INFORMATION", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"dwVersion": SimTypeInt(signed=False, label="UInt32"), "cbId": SimTypeInt(signed=False, label="UInt32"), "pbId": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "pwszName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pwszIcon": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pwszDisplayName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="WEBAUTHN_USER_ENTITY_INFORMATION", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"cCredentialParameters": SimTypeInt(signed=False, label="UInt32"), "pCredentialParameters": SimTypePointer(SimStruct({"dwVersion": SimTypeInt(signed=False, label="UInt32"), "pwszCredentialType": SimTypePointer(SimTypeChar(label="Char"), offset=0), "lAlg": SimTypeInt(signed=True, label="Int32")}, name="WEBAUTHN_COSE_CREDENTIAL_PARAMETER", pack=False, align=None), offset=0)}, name="WEBAUTHN_COSE_CREDENTIAL_PARAMETERS", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"dwVersion": SimTypeInt(signed=False, label="UInt32"), "cbClientDataJSON": SimTypeInt(signed=False, label="UInt32"), "pbClientDataJSON": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "pwszHashAlgId": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="WEBAUTHN_CLIENT_DATA", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"dwVersion": SimTypeInt(signed=False, label="UInt32"), "dwTimeoutMilliseconds": SimTypeInt(signed=False, label="UInt32"), "CredentialList": SimStruct({"cCredentials": SimTypeInt(signed=False, label="UInt32"), "pCredentials": SimTypePointer(SimStruct({"dwVersion": SimTypeInt(signed=False, label="UInt32"), "cbId": SimTypeInt(signed=False, label="UInt32"), "pbId": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "pwszCredentialType": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="WEBAUTHN_CREDENTIAL", pack=False, align=None), offset=0)}, name="WEBAUTHN_CREDENTIALS", pack=False, align=None), "Extensions": SimStruct({"cExtensions": SimTypeInt(signed=False, label="UInt32"), "pExtensions": SimTypePointer(SimStruct({"pwszExtensionIdentifier": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cbExtension": SimTypeInt(signed=False, label="UInt32"), "pvExtension": SimTypePointer(SimTypeBottom(label="Void"), offset=0)}, name="WEBAUTHN_EXTENSION", pack=False, align=None), offset=0)}, name="WEBAUTHN_EXTENSIONS", pack=False, align=None), "dwAuthenticatorAttachment": SimTypeInt(signed=False, label="UInt32"), "bRequireResidentKey": SimTypeInt(signed=True, label="Int32"), "dwUserVerificationRequirement": SimTypeInt(signed=False, label="UInt32"), "dwAttestationConveyancePreference": SimTypeInt(signed=False, label="UInt32"), "dwFlags": SimTypeInt(signed=False, label="UInt32"), "pCancellationId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0), "pExcludeCredentialList": SimTypePointer(SimStruct({"cCredentials": SimTypeInt(signed=False, label="UInt32"), "ppCredentials": SimTypePointer(SimTypePointer(SimStruct({"dwVersion": SimTypeInt(signed=False, label="UInt32"), "cbId": SimTypeInt(signed=False, label="UInt32"), "pbId": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "pwszCredentialType": SimTypePointer(SimTypeChar(label="Char"), offset=0), "dwTransports": SimTypeInt(signed=False, label="UInt32")}, name="WEBAUTHN_CREDENTIAL_EX", pack=False, align=None), offset=0), offset=0)}, name="WEBAUTHN_CREDENTIAL_LIST", pack=False, align=None), offset=0)}, name="WEBAUTHN_AUTHENTICATOR_MAKE_CREDENTIAL_OPTIONS", pack=False, align=None), offset=0), SimTypePointer(SimTypePointer(SimStruct({"dwVersion": SimTypeInt(signed=False, label="UInt32"), "pwszFormatType": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cbAuthenticatorData": SimTypeInt(signed=False, label="UInt32"), "pbAuthenticatorData": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "cbAttestation": SimTypeInt(signed=False, label="UInt32"), "pbAttestation": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "dwAttestationDecodeType": SimTypeInt(signed=False, label="UInt32"), "pvAttestationDecode": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "cbAttestationObject": SimTypeInt(signed=False, label="UInt32"), "pbAttestationObject": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "cbCredentialId": SimTypeInt(signed=False, label="UInt32"), "pbCredentialId": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "Extensions": SimStruct({"cExtensions": SimTypeInt(signed=False, label="UInt32"), "pExtensions": SimTypePointer(SimStruct({"pwszExtensionIdentifier": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cbExtension": SimTypeInt(signed=False, label="UInt32"), "pvExtension": SimTypePointer(SimTypeBottom(label="Void"), offset=0)}, name="WEBAUTHN_EXTENSION", pack=False, align=None), offset=0)}, name="WEBAUTHN_EXTENSIONS", pack=False, align=None), "dwUsedTransport": SimTypeInt(signed=False, label="UInt32")}, name="WEBAUTHN_CREDENTIAL_ATTESTATION", pack=False, align=None), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hWnd", "pRpInformation", "pUserInformation", "pPubKeyCredParams", "pWebAuthNClientData", "pWebAuthNMakeCredentialOptions", "ppWebAuthNCredentialAttestation"]),
        #
        'WebAuthNAuthenticatorGetAssertion': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimStruct({"dwVersion": SimTypeInt(signed=False, label="UInt32"), "cbClientDataJSON": SimTypeInt(signed=False, label="UInt32"), "pbClientDataJSON": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "pwszHashAlgId": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="WEBAUTHN_CLIENT_DATA", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"dwVersion": SimTypeInt(signed=False, label="UInt32"), "dwTimeoutMilliseconds": SimTypeInt(signed=False, label="UInt32"), "CredentialList": SimStruct({"cCredentials": SimTypeInt(signed=False, label="UInt32"), "pCredentials": SimTypePointer(SimStruct({"dwVersion": SimTypeInt(signed=False, label="UInt32"), "cbId": SimTypeInt(signed=False, label="UInt32"), "pbId": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "pwszCredentialType": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="WEBAUTHN_CREDENTIAL", pack=False, align=None), offset=0)}, name="WEBAUTHN_CREDENTIALS", pack=False, align=None), "Extensions": SimStruct({"cExtensions": SimTypeInt(signed=False, label="UInt32"), "pExtensions": SimTypePointer(SimStruct({"pwszExtensionIdentifier": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cbExtension": SimTypeInt(signed=False, label="UInt32"), "pvExtension": SimTypePointer(SimTypeBottom(label="Void"), offset=0)}, name="WEBAUTHN_EXTENSION", pack=False, align=None), offset=0)}, name="WEBAUTHN_EXTENSIONS", pack=False, align=None), "dwAuthenticatorAttachment": SimTypeInt(signed=False, label="UInt32"), "dwUserVerificationRequirement": SimTypeInt(signed=False, label="UInt32"), "dwFlags": SimTypeInt(signed=False, label="UInt32"), "pwszU2fAppId": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pbU2fAppId": SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), "pCancellationId": SimTypePointer(SimTypeBottom(label="Guid"), offset=0), "pAllowCredentialList": SimTypePointer(SimStruct({"cCredentials": SimTypeInt(signed=False, label="UInt32"), "ppCredentials": SimTypePointer(SimTypePointer(SimStruct({"dwVersion": SimTypeInt(signed=False, label="UInt32"), "cbId": SimTypeInt(signed=False, label="UInt32"), "pbId": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "pwszCredentialType": SimTypePointer(SimTypeChar(label="Char"), offset=0), "dwTransports": SimTypeInt(signed=False, label="UInt32")}, name="WEBAUTHN_CREDENTIAL_EX", pack=False, align=None), offset=0), offset=0)}, name="WEBAUTHN_CREDENTIAL_LIST", pack=False, align=None), offset=0)}, name="WEBAUTHN_AUTHENTICATOR_GET_ASSERTION_OPTIONS", pack=False, align=None), offset=0), SimTypePointer(SimTypePointer(SimStruct({"dwVersion": SimTypeInt(signed=False, label="UInt32"), "cbAuthenticatorData": SimTypeInt(signed=False, label="UInt32"), "pbAuthenticatorData": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "cbSignature": SimTypeInt(signed=False, label="UInt32"), "pbSignature": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "Credential": SimStruct({"dwVersion": SimTypeInt(signed=False, label="UInt32"), "cbId": SimTypeInt(signed=False, label="UInt32"), "pbId": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "pwszCredentialType": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="WEBAUTHN_CREDENTIAL", pack=False, align=None), "cbUserId": SimTypeInt(signed=False, label="UInt32"), "pbUserId": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="WEBAUTHN_ASSERTION", pack=False, align=None), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hWnd", "pwszRpId", "pWebAuthNClientData", "pWebAuthNGetAssertionOptions", "ppWebAuthNAssertion"]),
        #
        'WebAuthNFreeCredentialAttestation': SimTypeFunction([SimTypePointer(SimStruct({"dwVersion": SimTypeInt(signed=False, label="UInt32"), "pwszFormatType": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cbAuthenticatorData": SimTypeInt(signed=False, label="UInt32"), "pbAuthenticatorData": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "cbAttestation": SimTypeInt(signed=False, label="UInt32"), "pbAttestation": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "dwAttestationDecodeType": SimTypeInt(signed=False, label="UInt32"), "pvAttestationDecode": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "cbAttestationObject": SimTypeInt(signed=False, label="UInt32"), "pbAttestationObject": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "cbCredentialId": SimTypeInt(signed=False, label="UInt32"), "pbCredentialId": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "Extensions": SimStruct({"cExtensions": SimTypeInt(signed=False, label="UInt32"), "pExtensions": SimTypePointer(SimStruct({"pwszExtensionIdentifier": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cbExtension": SimTypeInt(signed=False, label="UInt32"), "pvExtension": SimTypePointer(SimTypeBottom(label="Void"), offset=0)}, name="WEBAUTHN_EXTENSION", pack=False, align=None), offset=0)}, name="WEBAUTHN_EXTENSIONS", pack=False, align=None), "dwUsedTransport": SimTypeInt(signed=False, label="UInt32")}, name="WEBAUTHN_CREDENTIAL_ATTESTATION", pack=False, align=None), offset=0)], SimTypeBottom(label="Void"), arg_names=["pWebAuthNCredentialAttestation"]),
        #
        'WebAuthNFreeAssertion': SimTypeFunction([SimTypePointer(SimStruct({"dwVersion": SimTypeInt(signed=False, label="UInt32"), "cbAuthenticatorData": SimTypeInt(signed=False, label="UInt32"), "pbAuthenticatorData": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "cbSignature": SimTypeInt(signed=False, label="UInt32"), "pbSignature": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "Credential": SimStruct({"dwVersion": SimTypeInt(signed=False, label="UInt32"), "cbId": SimTypeInt(signed=False, label="UInt32"), "pbId": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "pwszCredentialType": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="WEBAUTHN_CREDENTIAL", pack=False, align=None), "cbUserId": SimTypeInt(signed=False, label="UInt32"), "pbUserId": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="WEBAUTHN_ASSERTION", pack=False, align=None), offset=0)], SimTypeBottom(label="Void"), arg_names=["pWebAuthNAssertion"]),
        #
        'WebAuthNGetCancellationId': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Guid"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pCancellationId"]),
        #
        'WebAuthNCancelCurrentOperation': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Guid"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pCancellationId"]),
        #
        'WebAuthNGetErrorName': SimTypeFunction([SimTypeInt(signed=True, label="Int32")], SimTypePointer(SimTypeChar(label="Char"), offset=0), arg_names=["hr"]),
        #
        'WebAuthNGetW3CExceptionDOMError': SimTypeFunction([SimTypeInt(signed=True, label="Int32")], SimTypeInt(signed=True, label="Int32"), arg_names=["hr"]),
    }

lib.set_prototypes(prototypes)



import angr
from ...storage.file import SimFileDescriptor

import logging

l = logging.getLogger(name=__name__)


PROT_READ = 0x1  #    /* Page can be read.  */
PROT_WRITE = 0x2  #    /* Page can be written.  */
PROT_EXEC = 0x4  #    /* Page can be executed.  */
PROT_NONE = 0x0  #    /* Page can not be accessed.  */
MAP_SHARED = 0x01  #    /* Share changes.  */
MAP_PRIVATE = 0x02  #    /* Changes are private.  */
MAP_ANONYMOUS = 0x20  #    /* Don't use a file.  */
MAP_FIXED = 0x10  #    /* Interpret addr exactly.  */


class mmap(angr.SimProcedure):
    def run(self, addr, length, prot, flags, fd, offset):  # pylint:disable=arguments-differ,unused-argument
        # if self.state.solver.symbolic(flags) or self.state.solver.eval(flags) != 0x22:
        #   raise Exception("mmap with other than MAP_PRIVATE|MAP_ANONYMOUS unsupported")
        l.debug("mmap(%s, %s, %s, %s, %s, %s) = ...", addr, length, prot, flags, fd, offset)

        #
        # File descriptor sanity check
        #
        sim_fd = None
        if self.state.solver.is_false(fd[31:0] == -1):
            if self.state.solver.symbolic(fd):
                raise angr.errors.SimPosixError("Can't map a symbolic file descriptor!!")
            if self.state.solver.symbolic(offset):
                raise angr.errors.SimPosixError("Can't map with a symbolic offset!!")
            sim_fd = self.state.posix.get_fd(fd)
            if sim_fd is None:
                l.warning("Trying to map a non-exsitent fd")
                return -1
            if not isinstance(sim_fd, SimFileDescriptor) or sim_fd.file is None:
                l.warning("Trying to map fd not supporting mmap (maybe a SimFileDescriptorDuplex?)")
                return -1

        #
        # Length
        #

        if self.state.solver.symbolic(length):
            size = self.state.solver.max_int(length)
            if size > self.state.libc.max_variable_size:
                l.warning(
                    "mmap size requested of %d exceeds libc.max_variable_size. Using size %d instead.",
                    size,
                    self.state.libc.max_variable_size,
                )
                size = self.state.libc.max_variable_size
        else:
            size = self.state.solver.eval(length)

        #
        # Addr
        #

        # Not handling symbolic addr for now
        addrs = self.state.solver.eval_upto(addr, 2)
        if len(addrs) == 2:
            err = "Cannot handle symbolic addr argument for mmap."
            l.error(err)
            raise angr.errors.SimPosixError(err)

        addr = addrs[0]

        # Call is asking for system to provide an address
        if addr == 0:
            addr = self.allocate_memory(size)

        #
        # Flags
        #

        # Only want concrete flags
        flags = self.state.solver.eval_upto(flags, 2)

        if len(flags) == 2:
            err = "Cannot handle symbolic flags argument for mmap."
            l.error(err)
            raise angr.errors.SimPosixError(err)

        flags = flags[0]

        # Sanity check. All mmap must have exactly one of MAP_SHARED or MAP_PRIVATE
        if (flags & MAP_SHARED and flags & MAP_PRIVATE) or flags & (MAP_SHARED | MAP_PRIVATE) == 0:
            l.debug("... = -1 (bad flags)")
            return self.state.solver.BVV(-1, self.state.arch.bits)

        # Do region mapping
        while True:
            try:
                self.state.memory.map_region(addr, size, prot[2:0], init_zero=bool(flags & MAP_ANONYMOUS))
                l.debug("... = %#x", addr)
                break

            except angr.SimMemoryError:
                # This page is already mapped

                if flags & MAP_FIXED:
                    l.debug("... = -1 (MAP_FIXED failure)")
                    return self.state.solver.BVV(-1, self.state.arch.bits)

                # Can't give you that address. Find a different one and loop back around to try again.
                addr = self.allocate_memory(size)

        # If the mapping comes with a file descriptor
        if sim_fd:
            if not sim_fd.file.seekable:
                raise angr.errors.SimPosixError("Only support seekable SimFile at the moment.")

            prot = self.state.solver.eval_exact(prot, 1)[0]

            if prot & PROT_WRITE:
                l.warning("Trying to map a file descriptor backed by a file")
                l.warning("Updates to the mapping are not carried through to the underlying file")

            # read data
            saved_pos = sim_fd.tell()
            sim_fd.seek(self.state.solver.eval(offset), whence="start")
            data, _ = sim_fd.read_data(size)
            sim_fd.seek(saved_pos, whence="start")
            self.state.memory.store(addr, data)

        return addr

    def allocate_memory(self, size):
        addr = self.state.heap.mmap_base
        new_base = addr + size

        if new_base & 0xFFF:
            new_base = (new_base & ~0xFFF) + 0x1000

        self.state.heap.mmap_base = new_base

        return addr



#!/usr/bin/env python3
# pylint: disable=missing-class-docstring,no-self-use,line-too-long

import unittest

from angr import SimState, SimFile


class TestPosix(unittest.TestCase):
    def test_file_create(self):
        # Create a state first
        state = SimState(arch="AMD64", mode="symbolic")

        # Create a file
        fd = state.posix.open(b"test", 1)

        assert fd == 3

    def test_file_read(self):
        state = SimState(arch="AMD64", mode="symbolic")

        content = state.solver.BVV(0xBADF00D, 32)
        content_size = content.size() // 8

        fd = state.posix.open(b"test", 1)
        simfd = state.posix.get_fd(fd)
        simfd.write_data(content)
        simfd.seek(0)
        simfd.read(0xC0000000, content_size)

        data = state.memory.load(0xC0000000, content_size)

        assert data is content

    def test_file_seek(self):
        # TODO: Make this test more complete

        state = SimState(arch="AMD64", mode="symbolic")

        # Normal seeking
        fd = state.posix.open(b"test1", 1)
        simfd = state.posix.get_fd(fd)
        simfd.seek(0, "start")
        assert state.solver.is_true(simfd.tell() == 0)
        state.posix.close(fd)

        # TODO: test case: seek cannot go beyond the file size or current file pos

        # seek should not work for stdin/stdout/stderr
        assert state.solver.is_false(state.posix.get_fd(0).seek(0))
        assert state.solver.is_false(state.posix.get_fd(1).seek(0))
        assert state.solver.is_false(state.posix.get_fd(2).seek(0))

        # Seek from the end
        state.fs.insert("test2", SimFile(name="qwer", size=20))
        fd = state.posix.open(b"test2", 1)
        simfd = state.posix.get_fd(fd)
        simfd.seek(0, "end")
        assert state.solver.is_true(simfd.tell() == 20)
        state.posix.close(fd)

        # seek to a symbolic position (whence symbolic end)
        fd = state.posix.open(b"unknown_size", 1)
        simfd = state.posix.get_fd(fd)
        real_end = state.fs.get("unknown_size").size
        simfd.seek(0, "end")
        assert real_end is simfd.tell()
        state.posix.close(fd)


if __name__ == "__main__":
    unittest.main()



# Performance tests on concrete code execution without invoking Unicorn engine
# uses a stripped-down SimEngine to only test the essential pieces
# TODO also use a stripped-down memory

import os
import time

import angr
import claripy

# attempt to turn off claripy debug mode
if hasattr(claripy, "set_debug"):
    claripy.set_debug(False)

test_location = os.path.join(os.path.dirname(os.path.realpath(__file__)), "../../../binaries")


class SkinnyEngine(
    angr.engines.SimEngineFailure, angr.engines.SimEngineSyscall, angr.engines.HooksMixin, angr.engines.HeavyVEXMixin
):
    pass


arch = "x86_64"
b = angr.Project(os.path.join(test_location, "tests", arch, "perf_tight_loops"), auto_load_libs=False)
state = b.factory.full_init_state(
    plugins={"registers": angr.state_plugins.SimLightRegisters()}, remove_options={angr.sim_options.COPY_STATES}
)
engine = SkinnyEngine(b)


def main():
    simgr = b.factory.simgr(state)
    simgr.explore(engine=engine)


if __name__ == "__main__":
    tstart = time.time()
    main()
    tend = time.time()
    print("Elapsed: %f sec" % (tend - tstart))



#!/usr/bin/env python3
# pylint: disable=missing-class-docstring,no-self-use,line-too-long
__package__ = __package__ or "tests.knowledge_plugins"  # pylint:disable=redefined-builtin

import os
import unittest

import networkx

import angr

from ..common import bin_location


test_location = os.path.join(bin_location, "tests")


class TestKbPlugins(unittest.TestCase):
    def test_kb_plugins(self):
        p = angr.Project(os.path.join(test_location, "x86_64", "fauxware"), auto_load_libs=False)

        assert isinstance(p.kb.data, angr.knowledge_plugins.Data)
        assert isinstance(p.kb.functions, angr.knowledge_plugins.FunctionManager)
        assert isinstance(p.kb.variables, angr.knowledge_plugins.VariableManager)
        assert isinstance(p.kb.labels, angr.knowledge_plugins.Labels)
        assert isinstance(p.kb.comments, angr.knowledge_plugins.Comments)

        assert isinstance(p.kb.callgraph, networkx.Graph)
        assert isinstance(p.kb.resolved_indirect_jumps, dict)
        assert isinstance(p.kb.unresolved_indirect_jumps, set)

        assert dir(p.kb) is not None
        for plugin in [
            "data",
            "functions",
            "variables",
            "labels",
            "comments",
            "callgraph",
            "resolved_indirect_jumps",
            "unresolved_indirect_jumps",
        ]:
            assert plugin in dir(p.kb)

    def test_kb_plugins_typed(self):
        p = angr.Project(os.path.join(test_location, "x86_64", "fauxware"), auto_load_libs=False)

        for plugin in [
            angr.knowledge_plugins.Data,
            angr.knowledge_plugins.FunctionManager,
            angr.knowledge_plugins.VariableManager,
            angr.knowledge_plugins.Labels,
            angr.knowledge_plugins.Comments,
        ]:
            assert p.kb.get_knowledge(plugin) is None

        for plugin in [
            angr.knowledge_plugins.Data,
            angr.knowledge_plugins.FunctionManager,
            angr.knowledge_plugins.VariableManager,
            angr.knowledge_plugins.Labels,
            angr.knowledge_plugins.Comments,
        ]:
            assert isinstance(p.kb.request_knowledge(plugin), plugin)

        # The default plugins should have been instantiated by `request_knowledge`, and should now be available
        for plugin in [
            angr.knowledge_plugins.Data,
            angr.knowledge_plugins.FunctionManager,
            angr.knowledge_plugins.VariableManager,
            angr.knowledge_plugins.Labels,
            angr.knowledge_plugins.Comments,
        ]:
            assert isinstance(p.kb.request_knowledge(plugin), plugin)

        # Check that explicitly creating and registering new kind of plugin also works
        class TestPlugin(angr.knowledge_plugins.KnowledgeBasePlugin):
            def __init__(self, kb=None):
                self._kb = kb

        # Assert that unknown plugins return None when using "get_knowledge"
        assert p.kb.get_knowledge(TestPlugin) is None

        t = TestPlugin(p.kb)
        p.kb.register_plugin("test_plugin", t)

        assert p.kb.get_knowledge(TestPlugin) == t


if __name__ == "__main__":
    unittest.main()



import logging

from ..values import SimSootValue_ArrayBaseRef
from .base import SimSootExpr

l = logging.getLogger("angr.engines.soot.expressions.newarray")


class SimSootExpr_NewArray(SimSootExpr):
    def _execute(self):
        element_type = self.expr.base_type
        size = self._translate_expr(self.expr.size).expr
        self.expr = self.new_array(self.state, element_type, size)

    @staticmethod
    def new_array(state, element_type, size):
        """
        Allocates a new array in memory and returns the reference to the base.
        """
        size_bounded = SimSootExpr_NewArray._bound_array_size(state, size)
        # return the reference of the array base
        # => elements getting lazy initialized in the javavm memory
        return SimSootValue_ArrayBaseRef(
            heap_alloc_id=state.javavm_memory.get_new_uuid(), element_type=element_type, size=size_bounded
        )

    @staticmethod
    def _bound_array_size(state, array_size):
        # check if array size can exceed MAX_ARRAY_SIZE
        max_array_size = state.solver.BVV(state.javavm_memory.max_array_size, 32)
        size_stays_below_maximum = state.solver.eval_upto(max_array_size.SGE(array_size), 2)

        # overwrite size, if it *always* exceeds the maximum
        if True not in size_stays_below_maximum:
            l.warning(
                "Array size %s always exceeds maximum size. It gets overwritten with the maximum %s.",
                array_size,
                max_array_size,
            )
            return max_array_size

        # bound size, if it *can* exceeds the maximum
        if True in size_stays_below_maximum and False in size_stays_below_maximum:
            l.warning(
                "Array size %s can exceed maximum size. It gets bounded with the maximum %s.",
                array_size,
                max_array_size,
            )
            state.solver.add(max_array_size.UGE(array_size))

        return array_size



#!/usr/bin/env python3
# pylint:disable=missing-class-docstring,no-self-use
__package__ = __package__ or "tests.analyses.reaching_definitions"  # pylint:disable=redefined-builtin

from typing import TYPE_CHECKING
import os
from unittest import main, TestCase

import claripy

import angr
from angr.analyses.reaching_definitions import FunctionHandler
from angr.storage.memory_mixins.paged_memory.pages.multi_values import MultiValues
from angr.errors import SimMemoryMissingError

if TYPE_CHECKING:
    from angr.analyses.reaching_definitions.rd_state import ReachingDefinitionsState
    from angr.analyses.reaching_definitions import FunctionCallData

from ...common import bin_location


TESTS_LOCATION = os.path.join(bin_location, "tests")


def load_cstring_from_loader_memory(project, addr: int, as_str: bool = False):
    # This function loads a null-terminated string from the static memory region
    s = b""
    while True:
        char_addr = addr + len(s)
        try:
            v = project.loader.memory.load(char_addr, 1)
        except KeyError:
            break
        if v == b"\x00":
            break
        s += v
    return s.decode("utf-8") if as_str else (s + b"\x00")


def load_cstring_from_memory_definitions(ld, addr: int, as_str: bool = False):
    # This function loads a null-terminated string from the memory definitions
    s = b""
    while True:
        char_addr = addr + len(s)
        try:
            v = ld.memory.load(char_addr, 1).one_value().concrete_value
        except SimMemoryMissingError:
            break
        if v == 0:
            break
        s += v.to_bytes(1, "little")
    return s.decode("utf-8") if as_str else (s + b"\x00")


class CustomFunctionHandler(FunctionHandler):
    def __init__(self, project):
        self.project = project
        self.system_cmd = ""
        self.strcpy_addr = None
        self.sscanf_str_addr = None
        self.sscanf_str = None
        self.sscanf_fmtstr_addr = None
        self.sscanf_fmtstr = None
        self.sscanf_out_value = None
        self.malloc_sizes = []

    def handle_impl_malloc(self, state: "ReachingDefinitionsState", data: "FunctionCallData"):
        ((src_atom,),) = data.args_atoms

        src_value = state.get_values(src_atom).one_value().concrete_value
        self.malloc_sizes.append(src_value)
        data.depends(list(data.ret_atoms)[0], value=MultiValues(claripy.BVV(0x12345678, 64)))

    def handle_impl___isoc99_sscanf(self, state: "ReachingDefinitionsState", data: "FunctionCallData"):
        (str_atom,), (fmtstr_atom,), (out_atom,) = data.args_atoms[:3]

        # string
        str_addr = state.get_values(str_atom).one_value().concrete_value
        self.sscanf_str_addr = str_addr
        str_ = load_cstring_from_loader_memory(self.project, str_addr)
        self.sscanf_str = str_

        # format string: "%d"
        fmtstr_addr = state.get_values(fmtstr_atom).one_value().concrete_value
        self.sscanf_fmtstr_addr = fmtstr_addr
        fmtstr = load_cstring_from_loader_memory(self.project, str_addr)
        self.sscanf_fmtstr = fmtstr

        # out pointer
        (dst,) = state.pointer_to_atoms(state.get_values(out_atom), size=4, endness=state.arch.memory_endness)
        (src,) = state.pointer_to_atoms(state.get_values(str_atom), size=len(str_), endness="Iend_BE")
        self.sscanf_out_value = int(str_.strip(b"\x00"))
        data.depends(dst, src, value=MultiValues(claripy.BVV(self.sscanf_out_value, 32)))

    def handle_impl_strcpy(self, state: "ReachingDefinitionsState", data: "FunctionCallData"):
        (dst_atom,), (src_atom,) = data.args_atoms

        # Assume source is a constant string
        src_addr = state.get_values(src_atom).one_value().concrete_value
        self.strcpy_addr = src_addr
        src_str = load_cstring_from_loader_memory(self.project, src_addr)
        src_str_size = len(src_str)

        (dst,) = state.pointer_to_atoms(state.get_values(dst_atom), size=src_str_size, endness="Iend_BE")
        (src,) = state.pointer_to_atoms(state.get_values(src_atom), size=src_str_size, endness="Iend_BE")

        data.depends(dst, src, value=MultiValues(claripy.BVV(src_str)))

    def handle_impl_system(self, state: "ReachingDefinitionsState", data: "FunctionCallData"):
        (cmd_atom,) = data.args_atoms[0]
        cmd_addr = state.get_values(cmd_atom).one_value().concrete_value
        self.system_cmd = load_cstring_from_memory_definitions(state.live_definitions, cmd_addr, as_str=True)


class TestFunctionhandler(TestCase):
    def test_function_handler_depends_has_endness(self):
        filename = os.path.join(TESTS_LOCATION, "x86_64", "rda_function_handler")
        project = angr.Project(filename, auto_load_libs=False)
        _ = project.analyses.CFGFast()
        handler = CustomFunctionHandler(project)
        _ = project.analyses.ReachingDefinitions("main", function_handler=handler)

        assert handler.system_cmd == "ABCDEFGH"
        assert handler.sscanf_str == b"12345678\x00"
        assert handler.sscanf_out_value == 12345678
        assert handler.malloc_sizes == [20, 12345678]


if __name__ == "__main__":
    main()



from collections import defaultdict
from typing import Any, Dict, List, Callable, Optional, Generic, Type, TypeVar, Tuple, Set, TYPE_CHECKING, Union

import networkx

from .visitors.graph import NodeType
from ...sim_state import SimState
from ...errors import AngrForwardAnalysisError
from ...errors import AngrSkipJobNotice, AngrDelayJobNotice, AngrJobMergingFailureNotice, AngrJobWideningFailureNotice
from ...utils.algo import binary_insert
from .job_info import JobInfo, JobType, JobKey

if TYPE_CHECKING:
    from .visitors.graph import GraphVisitor

AnalysisState = TypeVar("AnalysisState")


class ForwardAnalysis(Generic[AnalysisState, NodeType, JobType, JobKey]):
    """
    This is my very first attempt to build a static forward analysis framework that can serve as the base of multiple
    static analyses in angr, including CFG analysis, VFG analysis, DDG, etc.

    In short, ForwardAnalysis performs a forward data-flow analysis by traversing a graph, compute on abstract values,
    and store results in abstract states. The user can specify what graph to traverse, how a graph should be traversed,
    how abstract values and abstract states are defined, etc.

    ForwardAnalysis has a few options to toggle, making it suitable to be the base class of several different styles of
    forward data-flow analysis implementations.

    ForwardAnalysis supports a special mode when no graph is available for traversal (for example, when a CFG is being
    initialized and constructed, no other graph can be used). In that case, the graph traversal functionality is
    disabled, and the optimal graph traversal order is not guaranteed. The user can provide a job sorting method to
    sort the jobs in queue and optimize traversal order.

    Feel free to discuss with me (Fish) if you have any suggestions or complaints.
    """

    def __init__(
        self,
        order_jobs=False,
        allow_merging=False,
        allow_widening=False,
        status_callback: Optional[Callable[[Type["ForwardAnalysis"]], Any]] = None,
        graph_visitor: "Optional[GraphVisitor[NodeType]]" = None,
    ):
        """
        Constructor

        :param bool order_jobs:     If all jobs should be ordered or not.
        :param bool allow_merging:  If job merging is allowed.
        :param bool allow_widening: If job widening is allowed.
        :param graph_visitor:       A graph visitor to provide successors.
        :type graph_visitor:        GraphVisitor or None
        :return: None
        """

        self._order_jobs = order_jobs
        self._allow_merging = allow_merging
        self._allow_widening = allow_widening
        self._status_callback = status_callback
        self._graph_visitor = graph_visitor

        # sanity checks
        if self._allow_widening and not self._allow_merging:
            raise AngrForwardAnalysisError("Merging must be allowed if widening is allowed.")

        # Analysis progress control
        self._should_abort = False

        # All remaining jobs
        self._job_info_queue: List[JobInfo[JobType, JobKey]] = []

        # A map between job key to job. Jobs with the same key will be merged by calling _merge_jobs()
        self._job_map: Dict[JobKey, JobInfo[JobType, JobKey]] = {}

        # A mapping between node and its input states
        self._input_states: Dict[NodeType, List[AnalysisState]] = defaultdict(list)
        # A mapping between node and its output state
        self._output_state: Dict[NodeType, AnalysisState] = {}

        # The graph!
        # Analysis results (nodes) are stored here
        self._graph = networkx.DiGraph()

    #
    # Properties
    #

    @property
    def should_abort(self):
        """
        Should the analysis be terminated.
        :return: True/False
        """

        return self._should_abort

    @property
    def graph(self) -> networkx.DiGraph:
        return self._graph

    @property
    def jobs(self):
        for job_info in self._job_info_queue:
            yield job_info.job

    #
    # Public methods
    #

    def abort(self):
        """
        Abort the analysis
        :return: None
        """

        self._should_abort = True

    def has_job(self, job: JobType) -> bool:
        """
        Checks whether there exists another job which has the same job key.
        :param job: The job to check.

        :return:    True if there exists another job with the same key, False otherwise.
        """
        job_key = self._job_key(job)
        return job_key in self._job_map

    def downsize(self):
        self._input_states = defaultdict(list)
        self._output_state = {}

    #
    # Abstract interfaces
    #

    # Common interfaces

    def _pre_analysis(self) -> None:
        raise NotImplementedError("_pre_analysis() is not implemented.")

    def _intra_analysis(self) -> None:
        raise NotImplementedError("_intra_analysis() is not implemented.")

    def _post_analysis(self) -> None:
        raise NotImplementedError("_post_analysis() is not implemented.")

    def _job_key(self, job: JobType) -> JobKey:
        raise NotImplementedError("_job_key() is not implemented.")

    def _get_successors(self, job: JobType) -> Union[List[SimState], List[JobType]]:
        raise NotImplementedError("_get_successors() is not implemented.")

    def _pre_job_handling(self, job: JobType) -> None:
        raise NotImplementedError("_pre_job_handling() is not implemented.")

    def _post_job_handling(self, job: JobType, new_jobs, successors: List[SimState]) -> None:
        raise NotImplementedError("_post_job_handling() is not implemented.")

    def _handle_successor(self, job: JobType, successor: SimState, successors: List[SimState]) -> List[JobType]:
        raise NotImplementedError("_handle_successor() is not implemented.")

    def _job_queue_empty(self) -> None:
        raise NotImplementedError("_job_queue_empty() is not implemented.")

    def _initial_abstract_state(self, node: NodeType) -> AnalysisState:
        raise NotImplementedError("_initial_abstract_state() is not implemented.")

    def _process_input_state_for_successor(
        self, node: NodeType, successor: NodeType, input_state: AnalysisState
    ) -> AnalysisState:
        # can be overwritten
        return input_state

    def _node_key(self, node: NodeType) -> NodeType:  # pylint:disable=no-self-use
        """
        Override this method if hash(node) is slow for the type of node that are in use.
        """
        return node

    def _run_on_node(self, node: NodeType, state: AnalysisState) -> Tuple[bool, AnalysisState]:
        """
        The analysis routine that runs on each node in the graph.

        For compatibility reasons, the variable `changed` in the returning tuple can have three values:
        - True, means a change has occurred, the output state is not the same as the input state, and a fixed point is
          not reached. Usually used if the analysis performs weak updates.
        - False, means no change has occurred. Usually used if the analysis performs weak updates.
        - None, means no change detection is performed during this process (e.g., if the analysis requires strong
          updates), and change detection will be performed later during _merge_states().

        :param node:    A node in the graph.
        :param state:   An abstract state that acts as the initial abstract state of this analysis routine.
        :return:        A tuple: (changed, output abstract state)
        """

        raise NotImplementedError("_run_on_node() is not implemented.")

    def _merge_states(self, node: NodeType, *states: AnalysisState) -> Tuple[AnalysisState, bool]:
        """
        Merge multiple abstract states into one.

        :param node:   A node in the graph.
        :param states: Abstract states to merge.
        :return:       A merged abstract state, and a boolean variable indicating if a local fixed-point has reached (
                       i.e., union(state0, state1) == state0), in which case, its successors will not be revisited.
        """

        raise NotImplementedError("_merge_states() is not implemented.")

    def _widen_states(self, *states: AnalysisState) -> AnalysisState:
        raise NotImplementedError("_widen_states() is not implemented.")

    # Special interfaces for non-graph-traversal mode

    def _merge_jobs(self, *jobs: JobType):
        raise NotImplementedError("_merge_jobs() is not implemented.")

    def _should_widen_jobs(self, *jobs: JobType):
        raise NotImplementedError("_should_widen_jobs() is not implemented.")

    def _widen_jobs(self, *jobs: JobType):
        raise NotImplementedError("_widen_jobs() is not implemented.")

    def _job_sorting_key(self, job: JobType) -> int:
        raise NotImplementedError("_job_sorting_key() is not implemented.")

    #
    # Private methods
    #

    def _analyze(self) -> None:
        """
        The main analysis routine.

        :return: None
        """

        self._pre_analysis()

        if self._graph_visitor is None:
            # There is no base graph that we can rely on. The analysis itself should generate successors for the
            # current job.
            # An example is the CFG recovery.

            self._analysis_core_baremetal()

        else:
            # We have a base graph to follow. Just handle the current job.

            self._analysis_core_graph()

        self._post_analysis()

    def _analysis_core_graph(self) -> None:
        while not self.should_abort:
            self._intra_analysis()

            n: NodeType = self._graph_visitor.next_node()

            if n is None:
                break

            job_state = self._get_and_update_input_state(n)
            if job_state is None:
                job_state = self._initial_abstract_state(n)

            changed, output_state = self._run_on_node(n, job_state)

            if changed is False:
                # no change is detected
                self._output_state[self._node_key(n)] = output_state
                continue
            if changed is True:
                # changes detected

                # output state of node n is input state for successors to node n
                self._add_input_state(n, output_state)

                # revisit all its successors
                self._graph_visitor.revisit_successors(n, include_self=False)
            else:
                # the change of states are determined during state merging (_add_input_state()) instead of during
                # simulated execution (_run_on_node()).

                if self._node_key(n) not in self._output_state:
                    reached_fixedpoint = False
                else:
                    # is the output state the same as the old one?
                    _, reached_fixedpoint = self._merge_states(n, self._output_state[self._node_key(n)], output_state)
                self._output_state[self._node_key(n)] = output_state

                if not reached_fixedpoint:
                    successors_to_visit = self._add_input_state(n, output_state)
                    # revisit all successors in the `successors_to_visit` list
                    for succ in successors_to_visit:
                        self._graph_visitor.revisit_node(succ)

    def _add_input_state(self, node: NodeType, input_state: AnalysisState) -> Set[NodeType]:
        """
        Add the input state to all successors of the given node.

        :param node:        The node whose successors' input states will be touched.
        :param input_state: The state that will be added to successors of the node.
        """

        successors = set(self._graph_visitor.successors(node))
        # successors_to_visit = set()  # a collection of successors whose input states did not reach a fixed point

        for succ in successors:
            # if a node has only one predecessor, we overwrite existing input states
            # otherwise, we add the state as a new input state
            # this is an approximation for removing input states for all nodes that `node` dominates
            new_input_state = self._process_input_state_for_successor(node, succ, input_state)
            if sum(1 for _ in self._graph_visitor.predecessors(succ)) == 1:
                self._input_states[self._node_key(succ)] = [new_input_state]
            else:
                self._input_states[self._node_key(succ)].append(new_input_state)

        return successors

    def _get_and_update_input_state(self, node: NodeType) -> Optional[AnalysisState]:
        """
        Get the input abstract state for this node, and remove it from the state map.

        :param node: The node in graph.
        :return:     A merged state, or None if there is no input state for this node available.
        """

        if self._node_key(node) in self._input_states:
            input_state = self._get_input_state(node)
            self._input_states[self._node_key(node)] = [input_state]
            return input_state
        return None

    def _get_input_state(self, node: NodeType) -> Optional[AnalysisState]:
        """
        Get the input abstract state for this node.

        :param node:    The node in graph.
        :return:        A merged state, or None if there is no input state for this node available.
        """

        if self._node_key(node) not in self._input_states:
            return None

        all_input_states = self._input_states.get(self._node_key(node))
        if len(all_input_states) == 1:
            return all_input_states[0]
        merged_state, _ = self._merge_states(node, *all_input_states)
        return merged_state

    def _analysis_core_baremetal(self) -> None:
        if not self._job_info_queue:
            self._job_queue_empty()

        while not self.should_abort:
            if self._status_callback is not None:
                self._status_callback(self)

            # should_abort might be changed by the status callback function
            if self.should_abort:
                return

            if not self._job_info_queue:
                self._job_queue_empty()

            if not self._job_info_queue:
                # still no job available
                break

            job_info = self._job_info_queue[0]

            try:
                self._pre_job_handling(job_info.job)
            except AngrDelayJobNotice:
                # delay the handling of this job
                continue
            except AngrSkipJobNotice:
                # consume and skip this job
                self._job_info_queue.pop(0)
                self._job_map.pop(self._job_key(job_info.job), None)
                continue

            # remove the job info from the map
            self._job_map.pop(self._job_key(job_info.job), None)
            self._job_info_queue.pop(0)

            self._process_job_and_get_successors(job_info)

            # Short-cut for aborting the analysis
            if self.should_abort:
                break

            self._intra_analysis()

    def _process_job_and_get_successors(self, job_info: JobInfo[JobType, JobKey]) -> None:
        """
        Process a job, get all successors of this job, and call _handle_successor() to handle each successor.

        :param JobInfo job_info: The JobInfo instance
        :return: None
        """

        job = job_info.job

        successors = self._get_successors(job)

        all_new_jobs = []

        for successor in successors:
            new_jobs = self._handle_successor(job, successor, successors)

            if new_jobs:
                all_new_jobs.extend(new_jobs)

                for new_job in new_jobs:
                    self._insert_job(new_job)

        self._post_job_handling(job, all_new_jobs, successors)

    def _insert_job(self, job: JobType) -> None:
        """
        Insert a new job into the job queue. If the job queue is ordered, this job will be inserted at the correct
        position.

        :param job: The job to insert
        :return:    None
        """

        key = self._job_key(job)

        if self._allow_merging:
            if key in self._job_map:
                job_info = self._job_map[key]

                # decide if we want to trigger a widening
                # if not, we'll simply do the merge
                # TODO: save all previous jobs for the sake of widening
                job_added = False
                if self._allow_widening and self._should_widen_jobs(job_info.job, job):
                    try:
                        widened_job = self._widen_jobs(job_info.job, job)
                        # remove the old job since now we have a widened one
                        if job_info in self._job_info_queue:
                            self._job_info_queue.remove(job_info)
                        job_info.add_job(widened_job, widened=True)
                        job_added = True
                    except AngrJobWideningFailureNotice:
                        # widening failed
                        # fall back to merging...
                        pass

                if not job_added:
                    try:
                        merged_job = self._merge_jobs(job_info.job, job)
                        # remove the old job since now we have a merged one
                        if job_info in self._job_info_queue:
                            self._job_info_queue.remove(job_info)
                        job_info.add_job(merged_job, merged=True)
                    except AngrJobMergingFailureNotice:
                        # merging failed
                        job_info = JobInfo(key, job)
                        # update the job map
                        self._job_map[key] = job_info

            else:
                job_info = JobInfo(key, job)
                self._job_map[key] = job_info

        else:
            job_info = JobInfo(key, job)
            self._job_map[key] = job_info

        if self._order_jobs:
            binary_insert(self._job_info_queue, job_info, lambda elem: self._job_sorting_key(elem.job))

        else:
            self._job_info_queue.append(job_info)

    def _peek_job(self, pos: int) -> JobType:
        """
        Return the job currently at position `pos`, but still keep it in the job queue. An IndexError will be raised
        if that position does not currently exist in the job list.

        :param int pos: Position of the job to get.
        :return:        The job
        """

        if pos < len(self._job_info_queue):
            return self._job_info_queue[pos].job

        raise IndexError()

    def _remove_job(self, predicate: Callable) -> None:
        """
        Remove jobs that satisfy certain criteria from job_info_queue.

        :param predicate:   A method that determines if a job should be removed or not.
        """

        to_remove = []
        for job_info in self._job_info_queue:
            if predicate(job_info.job):
                to_remove.append(job_info)

        for job_info in to_remove:
            self._job_info_queue.remove(job_info)
            key = self._job_key(job_info.job)
            if key in self._job_map:
                del self._job_map[key]



#!/usr/bin/env python3
__package__ = __package__ or "tests.analyses"  # pylint:disable=redefined-builtin

import time
import os
import logging
import unittest
import claripy

import angr

from ..common import bin_location


test_location = os.path.join(bin_location, "tests")
l = logging.getLogger("angr_tests")

vfg_buffer_overflow_addresses = {"x86_64": 0x40055C}

vfg_1_addresses = {
    "x86_64": {
        0x40071D,  # main
        0x400510,  # _puts
        0x40073E,  # main
        0x400530,  # _read
        0x400754,  # main
        0x40076A,  # main
        0x400774,  # main
        0x40078A,  # main
        0x4007A0,  # main
        0x400664,  # authenticate
        0x400550,  # _strcmp
        0x40068E,  # authenticate
        0x400699,  # authenticate
        0x400560,  # _open
        0x4006AF,  # authenticate
        0x4006C8,  # authenticate
        0x4006DB,  # authenticate
        0x400692,  # authenticate
        0x4006DF,  # authenticate
        0x4006E6,  # authenticate
        0x4006EB,  # authenticate
        0x4007BD,  # main
        0x4006ED,  # accepted
        0x4006FB,  # accepted
        0x4007C7,  # main
        0x4007C9,  # main
        0x4006FD,  # rejected
        0x400520,  # _printf
        0x400713,  # rejected
        0x400570,  # _exit
    }
}


class TestVfg(unittest.TestCase):
    def _run_vfg_buffer_overflow(self, arch):
        # pylint: disable=no-member
        proj = angr.Project(
            os.path.join(test_location, arch, "basic_buffer_overflows"),
            use_sim_procedures=True,
            default_analysis_mode="symbolic",
            auto_load_libs=False,
        )

        cfg = proj.analyses.CFGEmulated(context_sensitivity_level=1)

        # For this test case, OPTIMIZE_IR does not work due to the way we are widening the states: an index variable
        # directly goes to 0xffffffff, and when OPTIMIZE_IR is used, it does a signed comparison with 0x27, which
        # eventually leads to the merged index variable covers all negative numbers and [0, 27]. This analysis result is
        # correct but not accurate, and we suffer from it in this test case.
        # The ultimate solution is to widen more carefully, or implement lookahead widening support.
        # TODO: Solve this issue later

        start = time.time()
        function_start = vfg_buffer_overflow_addresses[arch]
        vfg = proj.analyses.VFG(
            cfg,
            function_start=function_start,
            context_sensitivity_level=2,
            interfunction_level=4,
            remove_options={angr.options.OPTIMIZE_IR},
        )
        end = time.time()
        duration = end - start

        l.info("VFG generation done in %f seconds.", duration)

        # TODO: These are very weak conditions. Make them stronger!
        assert len(vfg.final_states) > 0
        states = vfg.final_states
        assert len(states) == 2
        stack_check_fail = proj._extern_obj.get_pseudo_addr("symbol hook: __stack_chk_fail")
        assert {s.solver.eval_one(s.ip) for s in states} == {
            stack_check_fail,
            0x4005B4,
        }

        state = [s for s in states if s.solver.eval_one(s.ip) == 0x4005B4][0]
        assert claripy.backends.vsa.is_true(state.stack_read(12, 4) >= 0x28)

    def broken_vfg_buffer_overflow(self):
        # Test for running VFG on a single function
        self._run_vfg_buffer_overflow("x86_64")

    #
    # VFG test case 0
    #

    def test_vfg_0(self):
        self._run_vfg_0("x86_64")

    def _run_vfg_0(self, arch):
        proj = angr.Project(
            os.path.join(test_location, arch, "vfg_0"),
            load_options={"auto_load_libs": False},
        )

        cfg = proj.analyses.CFG(normalize=True)
        main = cfg.functions.function(name="main")
        vfg = proj.analyses.VFG(
            cfg,
            start=main.addr,
            context_sensitivity_level=1,
            interfunction_level=3,
            record_function_final_states=True,
            max_iterations=80,
        )

        function_final_states = vfg._function_final_states
        assert main.addr in function_final_states

        final_state_main = next(iter(function_final_states[main.addr].values()))
        stdout = final_state_main.posix.dumps(1)

        assert stdout[:6] == b"i = 64"
        # the following does not work without affine relation analysis
        # assert stdout == "i = 64

    #
    # VFG test case 1
    #

    def _run_vfg_1(self, arch):
        proj = angr.Project(
            os.path.join(test_location, arch, "fauxware"), use_sim_procedures=True, auto_load_libs=False
        )

        cfg = proj.analyses.CFGEmulated()
        vfg = proj.analyses.VFG(
            cfg,
            start=0x40071D,
            context_sensitivity_level=10,
            interfunction_level=10,
            record_function_final_states=True,
        )

        all_block_addresses = {n.addr for n in vfg.graph.nodes()}
        assert vfg_1_addresses[arch].issubset(all_block_addresses)

        # return value for functions

        # function authenticate has only two possible return values: 0 and 1
        authenticate = cfg.functions.function(name="authenticate")
        assert authenticate.addr in vfg.function_final_states
        authenticate_final_states = vfg.function_final_states[authenticate.addr]
        assert len(authenticate_final_states) == 1
        authenticate_final_state = next(iter(authenticate_final_states.values()))
        assert authenticate_final_state is not None
        assert authenticate_final_state.solver.eval_upto(authenticate_final_state.regs.rax, 3) == [0, 1]

        # optimal execution tests
        # - the basic block after returning from `authenticate` should only be executed once
        assert vfg._execution_counter[0x4007B3] == 1
        # - the last basic block in `authenticate` should only be executed once (on a non-normalized CFG)
        assert vfg._execution_counter[0x4006EB] == 1

    def test_vfg_1(self):
        # Test the code coverage of VFG
        self._run_vfg_1("x86_64")

    def test_vfg_resolving_indirect_calls(self):
        # resolving indirect calls provided via a statically allocated list of function addresses
        # the test binary is contributed by Luke Serné on angr Slack
        proj = angr.Project(
            os.path.join(test_location, "aarch64", "func-chain-aarch64"),
            auto_load_libs=False,
        )
        cfg = proj.analyses.CFG(normalize=True)
        vfg = proj.analyses.VFG(
            cfg,
            start=cfg.kb.functions["main"].addr,
            context_sensitivity_level=1,
            interfunction_level=1,
            record_function_final_states=True,
        )

        # find the node with indirect call exits
        expected_indirect_call_targets = {
            (0x400808, 0x400754),  # init_0
            (0x400808, 0x400770),  # init_1
            (0x400808, 0x400790),  # init_2
        }
        indirect_call_targets = set()
        for block_addr in cfg.kb.functions["main"].block_addrs_set:
            cfg_node = cfg.get_any_node(block_addr)
            succs_and_jumpkinds = cfg_node.successors_and_jumpkinds()
            if len(succs_and_jumpkinds) == 1 and succs_and_jumpkinds[0][1] == "Ijk_Call":
                # does it lead to an UnresolvedCall?
                if succs_and_jumpkinds[0][0].name == "UnresolvableCallTarget":
                    # found it!
                    for vfg_node in vfg.get_all_nodes(cfg_node.addr):
                        for successor_state in vfg_node.final_states:
                            if successor_state.history.jumpkind == "Ijk_Call":
                                indirect_call_targets.add((cfg_node.addr, successor_state.addr))

        assert expected_indirect_call_targets == indirect_call_targets


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python3
# Disable some pylint warnings: no-self-use, missing-docstring
# pylint: disable=R0201, C0111
__package__ = __package__ or "tests.procedures.posix"  # pylint:disable=redefined-builtin

import os
import unittest

import claripy

import angr

from ...common import bin_location


test_location = os.path.join(bin_location, "tests")


class TestRunEcho(unittest.TestCase):
    flag = "this_is_a_string!"

    def test_run_getenv_with_env(self):
        env = {"PATH": "/home/angr/", "TEST_ENV1": "this_is_a_string!", "JAVA_HOME": "jdk-install-dir"}
        TEST_name = ["TEST_ENV1", "TEST_ENV2"]
        p = angr.Project(os.path.join(test_location, "x86_64", "test_getenv"))
        s = p.factory.entry_state(env=env)
        simgr = p.factory.simulation_manager(s)
        simgr.explore()

        assert len(simgr.deadended) == 1

        output_lines = simgr.deadended[0].posix.dumps(1).decode().splitlines(keepends=False)
        expect_output = (
            [f"# {k}={v}" for k, v in env.items()]
            + ["{k}: {v}".format(k=k, v=env.get(k, "__NULL__")) for k in TEST_name]
            + ["!! Bingo " + self.flag]
        )

        output_lines.sort()
        expect_output.sort()

        assert output_lines == expect_output

    def test_run_getenv_without_env(self):
        p = angr.Project(os.path.join(test_location, "x86_64", "test_getenv"))
        s = p.factory.entry_state()
        simgr = p.factory.simulation_manager(s)
        simgr.explore()

        assert len(simgr.deadended) == 2

        bingo_count = 0
        for s in simgr.deadended:
            bingo_count += int(b"Bingo" in s.posix.dumps(1))

        assert bingo_count == 1

    def test_run_getenv_with_symbolic_env(self):
        flag = claripy.Concat(*[claripy.BVS("flag_%d" % i, 8) for i in range(30)])
        env = {"PATH": "/home/angr/", "TEST_ENV1": flag, "JAVA_HOME": "jdk-install-dir"}
        p = angr.Project(os.path.join(test_location, "x86_64", "test_getenv"))
        s = p.factory.entry_state(env=env)
        simgr = p.factory.simulation_manager(s)
        simgr.explore()

        assert len(simgr.deadended) == 2

        solved_flag = []
        for s in simgr.deadended:
            if b"Bingo" in s.posix.dumps(1):
                solved_flag.append(s.solver.eval(flag, cast_to=bytes).strip(b"\x00"))

        assert len(solved_flag) == 1
        assert solved_flag[0].decode() == self.flag


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python3
# pylint: disable=missing-class-docstring,no-self-use,line-too-long
__package__ = __package__ or "tests.analyses"  # pylint:disable=redefined-builtin

import sys
import platform
import os
import tempfile
import subprocess
import shutil
import unittest

import angr

from ..common import bin_location, has_32_bit_compiler_support


test_location = os.path.join(bin_location, "tests")

# Note: Reassembler is intensively tested by Patcherex test cases on CGC binaries.


def is_linux_x64():
    return sys.platform.startswith("linux") and platform.machine().endswith("64")


def is_linux():
    return sys.platform.startswith("linux")


class TestReassembler(unittest.TestCase):
    def test_data_reference_collection_in_add(self):
        # Issue reported and test binary provided by Antonio F. Montoya
        # Fixed in https://github.com/angr/pyvex/pull/192

        p = angr.Project(os.path.join(test_location, "x86_64", "df_gcc_-O1"), auto_load_libs=False)
        vexblock_opt0 = p.factory.block(0x402431, opt_level=0).vex
        vexblock_opt1 = p.factory.block(0x402431, opt_level=1).vex
        vexblock_opt1_nostmt = p.factory.block(0x402431, opt_level=1, collect_data_refs=True).vex_nostmt

        cfg = p.analyses.CFG()

        cfg._model.memory_data = {}
        cfg._collect_data_references(vexblock_opt0, 0x402431)
        memory_data_opt0 = cfg._model.memory_data

        cfg._model.memory_data = {}
        # bypass the IRSB unoptimization step
        cfg._collect_data_references_by_scanning_stmts(vexblock_opt1, 0x402431)
        memory_data_opt1 = cfg._model.memory_data

        cfg._model.memory_data = {}
        cfg._collect_data_references(vexblock_opt1_nostmt, 0x402431)
        memory_data_opt1_nostmt = cfg._model.memory_data

        assert memory_data_opt0.keys() == memory_data_opt1.keys()
        assert memory_data_opt0.keys() == memory_data_opt1_nostmt.keys()

    def test_ln_gcc_O2(self):
        # Issue reported and test binary provided by Antonio F. Montoya

        p = angr.Project(os.path.join(test_location, "x86_64", "ln_gcc_-O2"), auto_load_libs=False)
        r = p.analyses.Reassembler(syntax="at&t")
        r.symbolize()
        r.remove_unnecessary_stuff()
        assembly = r.assembly(comments=True, symbolized=True)

        # There should be two symbols with the same name: file_name. Reassembler renames the second one to file_name_0.
        # Test their existence.
        assert "\nfile_name:" in assembly and "\nfile_name_0:" in assembly

        if is_linux_x64():
            # we should be able to compile it and run it ... if we are running on x64 Linux
            tempdir = tempfile.mkdtemp(prefix="angr_test_reassembler_")
            asm_filename = "ln_gcc-O2.s"
            bin_filename = "ln_gcc-O2"
            asm_filepath = os.path.join(tempdir, asm_filename)
            bin_filepath = os.path.join(tempdir, bin_filename)
            with open(asm_filepath, "w", encoding="ascii") as f:
                f.write(assembly)
            # Call out to GCC, and it should return 0. Otherwise check_call() will raise an exception.
            subprocess.check_call(
                ["gcc", "-no-pie", asm_filepath, "-o", bin_filepath],
                stdout=subprocess.DEVNULL,
                stderr=subprocess.DEVNULL,
            )
            # Run the generated binary file, and it should not crash (which is a pretty basic requirement, I know)
            subprocess.check_call([bin_filepath, "--help"], stdout=subprocess.DEVNULL)
            # Pick up after ourselves
            shutil.rmtree(tempdir)

    def test_chmod_gcc_O1(self):
        # Issue reported and test binary provided by Antonio F. Montoya

        p = angr.Project(os.path.join(test_location, "x86_64", "chmod_gcc_-O1"), auto_load_libs=False)
        r = p.analyses.Reassembler(syntax="at&t")
        r.symbolize()
        r.remove_unnecessary_stuff()
        assembly = r.assembly(comments=True, symbolized=True)

        if is_linux_x64():
            # we should be able to compile it and run it ... if we are running on x64 Linux
            tempdir = tempfile.mkdtemp(prefix="angr_test_reassembler_")
            asm_filename = "chmod_gcc-O1.s"
            bin_filename = "chmod_gcc-O1"
            asm_filepath = os.path.join(tempdir, asm_filename)
            bin_filepath = os.path.join(tempdir, bin_filename)
            with open(asm_filepath, "w", encoding="ascii") as f:
                f.write(assembly)
            # Call out to GCC, and it should return 0. Otherwise check_call() will raise an exception.
            subprocess.check_call(
                ["gcc", "-no-pie", asm_filepath, "-o", bin_filepath],
                stdout=subprocess.DEVNULL,
                stderr=subprocess.DEVNULL,
            )
            # Run the generated binary file, and it should not crash (which is a pretty basic requirement, I know)
            subprocess.check_call([bin_filepath, "--help"], stdout=subprocess.DEVNULL)
            # Pick up after ourselves
            shutil.rmtree(tempdir)

    def test_ex_gpp(self):
        # Issue reported and test binary provided by Antonio F. Montoya

        p = angr.Project(os.path.join(test_location, "x86_64", "ex_g++"), auto_load_libs=False)
        r = p.analyses.Reassembler(syntax="at&t")
        r.symbolize()
        r.remove_unnecessary_stuff()
        assembly = r.assembly(comments=True, symbolized=True)

        if is_linux_x64():
            # we should be able to compile it and run it ... if we are running on x64 Linux
            tempdir = tempfile.mkdtemp(prefix="angr_test_reassembler_")
            asm_filename = "ex_g++.s"
            bin_filename = "ex_g++"
            asm_filepath = os.path.join(tempdir, asm_filename)
            bin_filepath = os.path.join(tempdir, bin_filename)
            with open(asm_filepath, "w", encoding="ascii") as f:
                f.write(assembly)
            # Call out to GCC, and it should return 0. Otherwise check_call() will raise an exception.
            subprocess.check_call(
                ["g++", "-no-pie", asm_filepath, "-o", bin_filepath],
                stdout=subprocess.DEVNULL,
                stderr=subprocess.DEVNULL,
            )
            # Run the generated binary file and check the output
            output = subprocess.check_output([bin_filepath])
            assert output == b"A1\nA2\n"
            # Pick up after ourselves
            shutil.rmtree(tempdir)

    def test_df_gcc_O1(self):
        # Issue reported and test binary provided by Antonio F. Montoya

        p = angr.Project(os.path.join(test_location, "x86_64", "df_gcc_-O1"), auto_load_libs=False)
        r = p.analyses.Reassembler(syntax="at&t")
        r.symbolize()
        r.remove_unnecessary_stuff()
        assembly = r.assembly(comments=True, symbolized=True)

        if is_linux_x64():
            # we should be able to compile it and run it ... if we are running on x64 Linux
            tempdir = tempfile.mkdtemp(prefix="angr_test_reassembler_")
            asm_filename = "df_gcc-O1.s"
            bin_filename = "df_gcc-O1"
            asm_filepath = os.path.join(tempdir, asm_filename)
            bin_filepath = os.path.join(tempdir, bin_filename)
            with open(asm_filepath, "w", encoding="ascii") as f:
                f.write(assembly)
            # Call out to GCC, and it should return 0. Otherwise check_call() will raise an exception.
            subprocess.check_call(
                ["gcc", "-no-pie", asm_filepath, "-o", bin_filepath],
                stdout=subprocess.DEVNULL,
                stderr=subprocess.DEVNULL,
            )
            # Run the generated binary file, and it should not crash (which is a pretty basic requirement, I know)
            subprocess.check_call([bin_filepath, "--help"], stdout=subprocess.DEVNULL)
            # Pick up after ourselves
            shutil.rmtree(tempdir)

    def test_dir_gcc_O0(self):
        # Issue reported and test binary provided by Antonio F. Montoya

        p = angr.Project(os.path.join(test_location, "x86_64", "dir_gcc_-O0"), auto_load_libs=False)
        r = p.analyses.Reassembler(syntax="at&t")
        r.symbolize()
        r.remove_unnecessary_stuff()
        assembly = r.assembly(comments=True, symbolized=True)

        if is_linux_x64():
            # we should be able to compile it and run it ... if we are running on x64 Linux
            tempdir = tempfile.mkdtemp(prefix="angr_test_reassembler_")
            asm_filename = "dir_gcc-O0.s"
            bin_filename = "dir_gcc-O0"
            asm_filepath = os.path.join(tempdir, asm_filename)
            bin_filepath = os.path.join(tempdir, bin_filename)
            with open(asm_filepath, "w", encoding="ascii") as f:
                f.write(assembly)
            # Call out to GCC, and it should return 0. Otherwise check_call() will raise an exception.
            subprocess.check_call(
                ["gcc", "-no-pie", asm_filepath, "-o", bin_filepath],
                stdout=subprocess.DEVNULL,
                stderr=subprocess.DEVNULL,
            )
            # Run the generated binary file, and it should not crash (which is a pretty basic requirement, I know)
            subprocess.check_call([bin_filepath, "--help"], stdout=subprocess.DEVNULL)
            subprocess.check_call([bin_filepath, "-la", "/"], stdout=subprocess.DEVNULL)
            # Pick up after ourselves
            shutil.rmtree(tempdir)

    def test_helloworld(self):
        # Reassembler complains about TYPE_OTHER symbols, which is because it's trying to classify bytes inside the ELF
        # header as pointers. We identify the ELF header in CFGFast to workaround this problem.
        # https://github.com/angr/angr/issues/1630

        p = angr.Project(os.path.join(test_location, "x86_64", "hello_world"), auto_load_libs=False)
        r = p.analyses.Reassembler(syntax="at&t")
        r.symbolize()
        r.remove_unnecessary_stuff()
        _ = r.assembly(comments=True, symbolized=True)

        # No exception should have been raised

    def test_helloworld_gcc9(self):
        # New versions of GCC changed the names of init and fini sections.
        # https://github.com/angr/patcherex/issues/39

        p = angr.Project(os.path.join(test_location, "x86_64", "hello_gcc9_reassembler"), auto_load_libs=False)
        r = p.analyses.Reassembler(syntax="at&t")
        r.symbolize()
        r.remove_unnecessary_stuff()
        assembly = r.assembly(comments=True, symbolized=True)

        if is_linux_x64():
            # we should be able to compile it and run it ... if we are running on x64 Linux
            tempdir = tempfile.mkdtemp(prefix="angr_test_reassembler_")
            asm_filename = "hello.s"
            bin_filename = "hello"
            asm_filepath = os.path.join(tempdir, asm_filename)
            bin_filepath = os.path.join(tempdir, bin_filename)
            with open(asm_filepath, "w", encoding="ascii") as f:
                f.write(assembly)
            # Call out to GCC, and it should return 0. Otherwise check_call() will raise an exception.
            subprocess.check_call(
                ["gcc", "-no-pie", asm_filepath, "-o", bin_filepath],
                stdout=subprocess.DEVNULL,
                stderr=subprocess.DEVNULL,
            )
            # Run the generated binary file, and it should not crash
            subprocess.check_call([bin_filepath], stdout=subprocess.DEVNULL)
            # Pick up after ourselves
            shutil.rmtree(tempdir)

    def test_partial_pie_ls_x86(self):
        # https://github.com/angr/patcherex/issues/39
        # a GCC-generated X86 binary with a few functions somehow being PIE

        p = angr.Project(os.path.join(test_location, "i386", "ls_gcc_7.5_reassembler"), auto_load_libs=False)
        r = p.analyses.Reassembler(syntax="at&t")
        r.symbolize()
        r.remove_unnecessary_stuff()
        assembly = r.assembly(comments=True, symbolized=True)

        if is_linux() and has_32_bit_compiler_support():
            # we should be able to compile it and run it ... if we are running on x64 Linux
            tempdir = tempfile.mkdtemp(prefix="angr_test_reassembler_")
            asm_filename = "ls.s"
            bin_filename = "ls"
            asm_filepath = os.path.join(tempdir, asm_filename)
            bin_filepath = os.path.join(tempdir, bin_filename)
            with open(asm_filepath, "w", encoding="ascii") as f:
                f.write(assembly)
            # Call out to GCC, and it should return 0. Otherwise check_call() will raise an exception.
            subprocess.check_call(
                ["gcc", "-m32", "-no-pie", asm_filepath, "-o", bin_filepath],
                stdout=subprocess.DEVNULL,
                stderr=subprocess.DEVNULL,
            )
            # Run the generated binary file, and it should not crash
            subprocess.check_call([bin_filepath], stdout=subprocess.DEVNULL)
            # We can also run it with "-h"
            o = subprocess.check_output([bin_filepath, "--version"])
            assert (
                o == b"ls (GNU coreutils) 8.30\n"
                b"Copyright (C) 2018 Free Software Foundation, Inc.\n"
                b"License GPLv3+: GNU GPL version 3 or later <https://gnu.org/licenses/gpl.html>.\n"
                b"This is free software: you are free to change and redistribute it.\n"
                b"There is NO WARRANTY, to the extent permitted by law.\n\n"
                b"Written by Richard M. Stallman and David MacKenzie.\n"
            )
            # Pick up after ourselves
            shutil.rmtree(tempdir)


if __name__ == "__main__":
    unittest.main()



# pylint:disable=line-too-long
import logging

from ...sim_type import SimTypeFunction,     SimTypeShort, SimTypeInt, SimTypeLong, SimTypeLongLong, SimTypeDouble, SimTypeFloat,     SimTypePointer,     SimTypeChar,     SimStruct,     SimTypeFixedSizeArray,     SimTypeBottom,     SimUnion,     SimTypeBool
from ...calling_conventions import SimCCStdcall, SimCCMicrosoftAMD64
from .. import SIM_PROCEDURES as P
from . import SimLibrary


_l = logging.getLogger(name=__name__)


lib = SimLibrary()
lib.set_default_cc('X86', SimCCStdcall)
lib.set_default_cc('AMD64', SimCCMicrosoftAMD64)
lib.set_library_names("wldp.dll")
prototypes = \
    {
        #
        'WldpGetLockdownPolicy': SimTypeFunction([SimTypePointer(SimStruct({"dwRevision": SimTypeInt(signed=False, label="UInt32"), "dwHostId": SimTypeInt(signed=False, label="WLDP_HOST_ID"), "szSource": SimTypePointer(SimTypeChar(label="Char"), offset=0), "hSource": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)}, name="WLDP_HOST_INFORMATION", pack=False, align=None), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["hostInformation", "lockdownState", "lockdownFlags"]),
        #
        'WldpIsClassInApprovedList': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimStruct({"dwRevision": SimTypeInt(signed=False, label="UInt32"), "dwHostId": SimTypeInt(signed=False, label="WLDP_HOST_ID"), "szSource": SimTypePointer(SimTypeChar(label="Char"), offset=0), "hSource": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)}, name="WLDP_HOST_INFORMATION", pack=False, align=None), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["classID", "hostInformation", "isApproved", "optionalFlags"]),
        #
        'WldpSetDynamicCodeTrust': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["fileHandle"]),
        #
        'WldpIsDynamicCodePolicyEnabled': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["isEnabled"]),
        #
        'WldpQueryDynamicCodeTrust': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["fileHandle", "baseImage", "imageSize"]),
    }

lib.set_prototypes(prototypes)



import angr


class __ctype_b_loc(angr.SimProcedure):
    """
    Following is the description from linuxfoundation.org:

    The __ctype_b_loc() function shall return a pointer into an array of
    characters in the current locale that contains characteristics for each
    character in the current character set. The array shall contain a total
    of 384 characters, and can be indexed with any signed or unsigned char
    (i.e. with an index value between -128 and 255). If the application is
    multithreaded, the array shall be local to the current thread.

    This interface is not in the source standard; it is only in the binary
    standard.
    """

    def run(self):
        table_ptr = self.state.libc.ctype_b_loc_table_ptr

        return table_ptr



#!/usr/bin/env python3
# pylint: disable=missing-class-docstring,disable=no-self-use
__package__ = __package__ or "tests.analyses"  # pylint:disable=redefined-builtin

import os
import unittest

import angr
from angr.analyses.code_tagging import CodeTags

from ..common import bin_location


test_location = os.path.join(bin_location, "tests")


class TestCodetagging(unittest.TestCase):
    def test_hasxor(self):
        p = angr.Project(os.path.join(test_location, "x86_64", "HashTest"), auto_load_libs=False)
        cfg = p.analyses.CFG()

        ct_rshash = p.analyses.CodeTagging(cfg.kb.functions["RSHash"])
        assert CodeTags.HAS_XOR not in ct_rshash.tags
        ct_jshash = p.analyses.CodeTagging(cfg.kb.functions["JSHash"])
        assert CodeTags.HAS_XOR in ct_jshash.tags
        assert CodeTags.HAS_BITSHIFTS in ct_jshash.tags
        ct_elfhash = p.analyses.CodeTagging(cfg.kb.functions["ELFHash"])
        assert CodeTags.HAS_BITSHIFTS in ct_elfhash.tags


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python3
# pylint: disable=missing-class-docstring,disable=no-self-use
__package__ = __package__ or "tests.analyses"  # pylint:disable=redefined-builtin

import logging
import os
import unittest

import angr

from ..common import bin_location


class TestIdentifier(unittest.TestCase):
    def test_comparison_identification(self):
        true_symbols = {0x804A3D0: "strncmp", 0x804A0F0: "strcmp", 0x8048E60: "memcmp", 0x8049F40: "strcasecmp"}

        p = angr.Project(os.path.join(bin_location, "tests", "i386", "identifiable"))
        idfer = p.analyses.Identifier(require_predecessors=False)

        seen = {}
        for addr, symbol in idfer.run():
            seen[addr] = symbol

        for addr, symbol in true_symbols.items():
            assert symbol == seen[addr]


if __name__ == "__main__":
    logging.getLogger("identifier").setLevel("DEBUG")
    unittest.main()



#!/usr/bin/env python3
import unittest

from angr import SIM_PROCEDURES
from angr import SimState, SimPosixError, SimFile


FAKE_ADDR = 0x100000


def lseek(state, arguments):
    return SIM_PROCEDURES["linux_kernel"]["lseek"]().execute(state, arguments=arguments)


# Taken from unistd.h
SEEK_SET = 0  # Seek from beginning of file.
SEEK_CUR = 1  # Seek from current position.
SEEK_END = 2  # Seek from end of file.
# GNU Extensions
SEEK_DATA = 3  # Seek to next data.
SEEK_HOLE = 4  # Seek to next hole.


class TestLseek(unittest.TestCase):
    def test_lseek_set(self):
        state = SimState(arch="AMD64", mode="symbolic")

        # This could be any number above 2 really
        fd = 3

        # Create a file
        state.fs.insert("/tmp/qwer", SimFile(name="qwer", size=100))
        assert fd == state.posix.open(b"/tmp/qwer", 2)

        # Part 1

        # Seek to the top of the file
        current_pos = lseek(state, [fd, 0, SEEK_SET]).ret_expr
        current_pos = state.solver.eval(current_pos)

        # We should be at the start
        assert current_pos == 0

        # Part 2

        # Seek to the top of the file
        current_pos = lseek(state, [fd, 8, SEEK_SET]).ret_expr
        current_pos = state.solver.eval(current_pos)

        # We should be at the start
        assert current_pos == 8

        # Part 3

        # Seek to the top of the file
        current_pos = lseek(state, [fd, 3, SEEK_SET]).ret_expr
        current_pos = state.solver.eval(current_pos)

        # We should be at the start
        assert current_pos == 3

    def test_lseek_cur(self):
        state = SimState(arch="AMD64", mode="symbolic")

        # This could be any number above 2 really
        fd = 3

        # Create a file
        state.fs.insert("/tmp/qwer", SimFile(name="qwer", size=100))
        assert fd == state.posix.open(b"/tmp/qwer", 2)

        # Part 1

        # Add 12
        current_pos = lseek(state, [fd, 12, SEEK_CUR]).ret_expr
        current_pos = state.solver.eval(current_pos)

        # We should be at the start
        assert current_pos == 12

        # Part 2

        # Remove 3
        current_pos = lseek(state, [fd, -3, SEEK_CUR]).ret_expr
        current_pos = state.solver.eval(current_pos)

        # We should be at the start
        assert current_pos == 9

    def test_lseek_end(self):
        state = SimState(arch="AMD64", mode="symbolic")

        fd = 3

        # Create a file
        state.fs.insert("/tmp/qwer", SimFile(name="qwer", size=16))
        assert fd == state.posix.open(b"/tmp/qwer", 2)

        # Part 1

        # Add 5
        current_pos = lseek(state, [fd, 0, SEEK_END]).ret_expr
        current_pos = state.solver.eval(current_pos)

        # We should be at the end + offset
        assert current_pos == 16

        # Part 2

        # Minus 6. End of file never actually changed
        current_pos = lseek(state, [fd, -6, SEEK_END]).ret_expr
        current_pos = state.solver.eval(current_pos)

        # We should be at the end + offset
        assert current_pos == 10

    def test_lseek_unseekable(self):
        state = SimState(arch="AMD64", mode="symbolic")

        # Illegal seek
        current_pos = lseek(state, [0, 0, SEEK_SET]).ret_expr
        current_pos = state.solver.eval(current_pos)

        # Assert we have a negative return value
        assert current_pos & (1 << 63) != 0

        # Illegal seek
        current_pos = lseek(state, [1, 0, SEEK_SET]).ret_expr
        current_pos = state.solver.eval(current_pos)

        # Assert we have a negative return value
        assert current_pos & (1 << 63) != 0

        # Illegal seek
        current_pos = lseek(state, [2, 0, SEEK_SET]).ret_expr
        current_pos = state.solver.eval(current_pos)

        # Assert we have a negative return value
        assert current_pos & (1 << 63) != 0

    def test_lseek_symbolic_whence(self):
        with self.assertRaises(SimPosixError):
            # symbolic whence is currently not possible
            state = SimState(arch="AMD64", mode="symbolic")

            # This could be any number above 2 really
            fd = 3

            # Create a file
            assert fd == state.posix.open(b"/tmp/qwer", 1)

            whence = state.solver.BVS("whence", 64)

            # This should cause the exception
            lseek(state, [fd, 0, whence])

    def test_lseek_symbolic_seek(self):
        # symbolic seek is currently not possible
        state = SimState(arch="AMD64", mode="symbolic")

        # This could be any number above 2 really
        fd = 3

        # Create a file
        assert fd == state.posix.open(b"/tmp/qwer", 1)

        seek = state.solver.BVS("seek", 64)

        # This should NOT cause an exception
        lseek(state, [fd, seek, SEEK_SET])


if __name__ == "__main__":
    unittest.main()
